@@ -1,12 +1,14 @@
 (**************************************************************************)
 (*                                                                        *)
-(*     The Alt-ergo theorem prover                                        *)
-(*     Copyright (C) 2006-2010                                            *)
+(*     The Alt-Ergo theorem prover                                        *)
+(*     Copyright (C) 2006-2011                                            *)
 (*                                                                        *)
 (*     Sylvain Conchon                                                    *)
 (*     Evelyne Contejean                                                  *)
-(*     Stephane Lescuyer                                                  *)
+(*                                                                        *)
+(*     Francois Bobot                                                     *)
 (*     Mohamed Iguernelala                                                *)
+(*     Stephane Lescuyer                                                  *)
 (*     Alain Mebsout                                                      *)
 (*                                                                        *)
 (*     CNRS - INRIA - Universite Paris Sud                                *)
@@ -198,7 +200,8 @@ module Make
         in 
         P.create l c ty
 *)
-  let make t = 
+  let make t =
+    if rules = 4 then fprintf fmt "[rule] TR-Arith-Make@.";
     let {T.ty = ty} = T.view t in
     let p, ctx = mke (Int 1) (empty_polynome ty) t [] in
     is_mine (arith_to_ac p), ctx
@@ -291,7 +294,9 @@ module Make
     is_mine p
 
 
-  let compare = P.compare
+  let compare_mine = P.compare
+
+  let compare x y = P.compare (embed x) (embed y)
 
   let hash = P.hash
 
@@ -468,7 +473,8 @@ module Make
     let pp = safe_distribution p in
     if ty = Ty.Treal then solve_real pp else solve_int pp
 
-  let solve r1 r2 = 
+  let solve r1 r2 =
+    if rules = 4 then fprintf fmt "[rule] TR-Arith-Solve@.";
     let sbs = solve_aux r1 r2 in
     let sbs = List.fast_sort (fun (a,_) (x,y) -> X.compare x a)sbs in
     if debug_arith then begin
@@ -489,6 +495,8 @@ module Make
       | _ -> false
 
 
+  let term_extract _ = None
+
   module Rel = Fm.Make (X) 
     (struct
        include P 
