@@ -1,12 +1,14 @@
 (**************************************************************************)
 (*                                                                        *)
-(*     The Alt-ergo theorem prover                                        *)
-(*     Copyright (C) 2006-2010                                            *)
+(*     The Alt-Ergo theorem prover                                        *)
+(*     Copyright (C) 2006-2011                                            *)
 (*                                                                        *)
 (*     Sylvain Conchon                                                    *)
 (*     Evelyne Contejean                                                  *)
-(*     Stephane Lescuyer                                                  *)
+(*                                                                        *)
+(*     Francois Bobot                                                     *)
 (*     Mohamed Iguernelala                                                *)
+(*     Stephane Lescuyer                                                  *)
 (*     Alain Mebsout                                                      *)
 (*                                                                        *)
 (*     CNRS - INRIA - Universite Paris Sud                                *)
@@ -17,6 +19,7 @@
 
 open Options
 open Format
+open Sig
   
 module Sy = Symbols
 module T  = Term
@@ -87,8 +90,7 @@ module Make(X : ALIEN) = struct
       let add k v ex mp = add k (Conseq.add (v,ex) (find k mp)) mp
     end
 
-    (* ensemble de termes "get" avec leurs arguments et leurs types*)
-    type gtype = {g:T.t; gt:T.t; gi:T.t; gty:Ty.t}
+    type gtype = {g:Term.t; gt:Term.t; gi:Term.t; gty:Ty.t}
     module G :Set.S with type elt = gtype = Set.Make
       (struct type t = gtype let compare t1 t2 = T.compare t1.g t2.g end)
 
@@ -110,9 +112,10 @@ module Make(X : ALIEN) = struct
     type t = 
         {gets  : G.t;               (* l'ensemble des "get" croises*)
          tbset : S.t TBS.t ;        (* map t |-> set(t,-,-) *)
-         split : LRset.t;            (* l'ensemble des case-split possibles *)
+         split : LRset.t;           (* l'ensemble des case-split possibles *)
          conseq   : Conseq.t LRmap.t; (* consequences des splits *)
-         seen  : T.Set.t Tmap.t     (* combinaisons (get,set) deja splitees *) }
+         seen  : T.Set.t Tmap.t;    (* combinaisons (get,set) deja splitees *)
+	}
           
 
     let empty _ = 
@@ -330,31 +333,35 @@ module Make(X : ALIEN) = struct
       in
       let eqs = extensionality eqs la class_of in
       implied_consequences env eqs la
-        
-
-    (* instantiation des axiomes des tableaux *)
-    let instantiate env are_eq are_dist class_of la =
-      Debug.assume fmt la; 
-      let env = new_terms env la in
-      let env, atoms = new_splits are_eq are_dist env Conseq.empty class_of in
-      let env, atoms = new_equalities env atoms la class_of in
-      Debug.env fmt env;
-      Debug.new_equalities fmt atoms;
-      env, Conseq.elements atoms 
-        
+       
+    (* XXXXXX : TODO -> ajouter les explications dans les choix du
+       case split *)
     (* choisir une egalite sur laquelle on fait un case-split *)
     let case_split env = 
       try
         let a = LRset.choose env.split in
         if debug_arrays then 
           fprintf fmt "[Arrays.case-split] %a@." LR.print a;
-        [(LR.view a, None), (Num.Int 2)]
+        [LR.view a, Ex.empty, Num.Int 2] 
       with Not_found ->
-        if debug_arrays then fprintf fmt "[Arrays.case-split] Nothing@.";
-        []
+	if debug_arrays then fprintf fmt "[Arrays.case-split] Nothing@.";
+	[]
           
-    let assume env _ _ = env, []
-    let query a _ _ = Sig.No
+    let assume env la ~are_eq ~are_neq ~class_of = 
+      (* instantiation des axiomes des tableaux *)
+      Debug.assume fmt la; 
+       let env = new_terms env la in
+      let env, atoms = new_splits are_eq are_neq env Conseq.empty class_of in
+      let env, atoms = new_equalities env atoms la class_of in
+      (*Debug.env fmt env;*)
+      Debug.new_equalities fmt atoms;
+      let l = Conseq.fold (fun (a,ex) l -> ((LTerm a, ex)::l)) atoms [] in
+      env, { assume = l; remove = [] }
+	  
+    let query _ _ ~are_eq ~are_neq ~class_of = Sig.No
     let add env r = env
+
   end
+
+  let term_extract _ = None
 end
