@@ -1,12 +1,14 @@
 (**************************************************************************)
 (*                                                                        *)
-(*     The Alt-ergo theorem prover                                        *)
-(*     Copyright (C) 2006-2010                                            *)
+(*     The Alt-Ergo theorem prover                                        *)
+(*     Copyright (C) 2006-2011                                            *)
 (*                                                                        *)
 (*     Sylvain Conchon                                                    *)
 (*     Evelyne Contejean                                                  *)
-(*     Stephane Lescuyer                                                  *)
+(*                                                                        *)
+(*     Francois Bobot                                                     *)
 (*     Mohamed Iguernelala                                                *)
+(*     Stephane Lescuyer                                                  *)
 (*     Alain Mebsout                                                      *)
 (*                                                                        *)
 (*     CNRS - INRIA - Universite Paris Sud                                *)
@@ -16,7 +18,7 @@
 (**************************************************************************)
 
 open Format
-open Hashcons
+open Sig
 
 module Sy = Symbols
 module T = Term
@@ -230,8 +232,8 @@ module Make(X : ALIEN) = struct
 	    begin 
 	      match T.view ti , T.view tj with
 		| { T.f = Sy.Int i } , { T.f = Sy.Int j } -> 
-		    let i = int_of_string i.node in
-		    let j = int_of_string j.node in
+		    let i = int_of_string i.Hashcons.node in
+		    let j = int_of_string j.Hashcons.node in
 		    let r1, ctx = make_rec t1 ctx in
 		    { sz = j - i + 1 ; bv = I_Ext (r1,i,j)}, ctx
 		| _ -> assert false
@@ -608,7 +610,7 @@ module Make(X : ALIEN) = struct
 
   end
 
-  let compare b1 b2 = 
+  let compare_mine b1 b2 = 
     let rec comp l1 l2 = match l1,l2 with
 	[] , [] -> 0
       | [] , _ -> -1
@@ -618,6 +620,8 @@ module Make(X : ALIEN) = struct
 	  if c<>0 then c else comp l1 l2
     in comp b1 b2
 
+  let compare x y = compare (embed x) (embed y)
+
   let hash_xterm = function
     | Var {var = i; sorte = A} -> 11 * i
     | Var {var = i; sorte = B} -> 17 * i
@@ -750,28 +754,30 @@ module Make(X : ALIEN) = struct
   module Map = Map.Make 
     (struct 
        type t = (X.r simple_term) list
-       let compare = compare 
+       let compare = compare_mine
      end)
     
   module Set = Set.Make (
     struct 
       type t = (X.r simple_term) list
-      let compare = compare
+      let compare = compare_mine
     end)
 
   let fully_interpreted sb = true
 
+  let term_extract _ = None
+
   module Rel = struct
 
     type r = X.r
     type t =  unit
 
     let empty () = ()
-    let assume _ _ _ = (), []
+    let assume _ _ ~are_eq ~are_neq ~class_of =
+      (), { assume = []; remove = [] }
     let add _ _ = ()
     let case_split _ = []    
-    let query _ _ _ = Sig.No
-    let instantiate _ _ _ _ _ = (), []
+    let query _ _ ~are_eq ~are_neq ~class_of = Sig.No
   end
 
 end
