@@ -1,12 +1,14 @@
 (**************************************************************************)
 (*                                                                        *)
-(*     The Alt-ergo theorem prover                                        *)
-(*     Copyright (C) 2006-2010                                            *)
+(*     The Alt-Ergo theorem prover                                        *)
+(*     Copyright (C) 2006-2011                                            *)
 (*                                                                        *)
 (*     Sylvain Conchon                                                    *)
 (*     Evelyne Contejean                                                  *)
-(*     Stephane Lescuyer                                                  *)
+(*                                                                        *)
+(*     Francois Bobot                                                     *)
 (*     Mohamed Iguernelala                                                *)
+(*     Stephane Lescuyer                                                  *)
 (*     Alain Mebsout                                                      *)
 (*                                                                        *)
 (*     CNRS - INRIA - Universite Paris Sud                                *)
@@ -40,9 +42,8 @@ module rec CX : sig
 end =
 struct
 
-  (* Xi < Term < Ac *)
   type r =
-    | Term  of Term.t (* XXX changement de l'ordre pour Arrays *)
+    | Term  of Term.t
     | Ac    of AC.t
     | X1    of X1.t 
     | X2    of X2.t 
@@ -77,26 +78,65 @@ struct
     in 
     ty = Ty.Tint
       
-  let rec compare a b = 
-    let c = compare_tag a b in 
-    if c = 0 then comparei a b else c
+  let type_info = function
+    | X1 t   -> X1.type_info t
+    | X2 t   -> X2.type_info t
+    | X3 t   -> X3.type_info t
+    | X4 t   -> X4.type_info t
+    | X5 t   -> X5.type_info t
+    | Ac x   -> AC.type_info x
+    | Term t -> let {Term.ty = ty} = Term.view t in ty
 
-  and compare_tag a b = 
-    Pervasives.compare (theory_num a) (theory_num b)
-      
-  and comparei a b = 
-    match a, b with
-      | X1 x, X1 y -> X1.compare x y
-      | X2 x, X2 y -> X2.compare x y
-      | X3 x, X3 y -> X3.compare x y
-      | X4 x, X4 y -> X4.compare x y
-      | X5 x, X5 y -> X5.compare x y
+  (* Xi < Term < Ac *)
+  let theory_num x = match x with  
+    | Ac _    -> -1
+    | Term  _ -> -2
+    | X1 _    -> -3
+    | X2 _    -> -4
+    | X3 _    -> -5
+    | X4 _    -> -6
+    | X5 _    -> -7
+          
+  let compare_tag a b = theory_num a - theory_num b
+
+  let compare a b = 
+    let c = Ty.compare (type_info a) (type_info b) in
+    if c <> 0 then c
+    else match a, b with
       | Term x  , Term y  -> Term.compare x y
       | Ac x    , Ac    y -> AC.compare x y
-      | _                 -> assert false
-
-  and theory_num x = Obj.tag (Obj.repr x)
-
+      | Ac _    , Term _  -> 1
+      | Term _  , Ac _    -> -1
+      | X1 _, (Term _ | Ac _ | X1 _) | (Term _ | Ac _ ), X1 _ -> X1.compare a b
+      | X2 _, (Term _ | Ac _ | X2 _) | (Term _ | Ac _ ), X2 _ -> X2.compare a b
+      | X3 _, (Term _ | Ac _ | X3 _) | (Term _ | Ac _ ), X3 _ -> X3.compare a b
+      | X4 _, (Term _ | Ac _ | X4 _) | (Term _ | Ac _ ), X4 _ -> X4.compare a b
+      | X5 _, (Term _ | Ac _ | X5 _) | (Term _ | Ac _ ), X5 _ -> X5.compare a b
+      | _                 -> compare_tag a b
+          
+    
+  (* ancienne version 
+     let rec compare a b = 
+     let c = compare_tag a b in 
+     if c = 0 then comparei a b else c
+     
+     and compare_tag a b = 
+     Pervasives.compare (theory_num a) (theory_num b)
+     
+     and comparei a b = 
+     match a, b with
+     | X1 x, X1 y -> X1.compare a b
+     | X2 x, X2 y -> X2.compare a b
+     | X3 x, X3 y -> X3.compare a b
+     | X4 x, X4 y -> X4.compare a b
+     | X5 x, X5 y -> X5.compare a b
+     | Term x  , Term y  -> Term.compare x y
+     | Ac x    , Ac    y -> AC.compare x y
+     | _                 -> assert false
+     
+     and theory_num x = Obj.tag (Obj.repr x)
+  *)
+    
   let equal a b = compare a b = 0
 
   let hash = function
@@ -111,17 +151,7 @@ struct
   module MR = Map.Make(struct type t = r let compare = compare end)
     
   let print fmt r = 
-    if verbose then 
-      match r with
-        | X1 t    -> fprintf fmt "X1(%s):[%a]" X1.name X1.print t
-        | X2 t    -> fprintf fmt "X2(%s):[%a]" X2.name X2.print t
-        | X3 t    -> fprintf fmt "X3(%s):[%a]" X3.name X3.print t
-        | X4 t    -> fprintf fmt "X3(%s):[%a]" X4.name X4.print t
-        | X5 t    -> fprintf fmt "X3(%s):[%a]" X5.name X5.print t
-        | Term t  -> fprintf fmt "FT:[%a]" Term.print t
-        | Ac t    -> fprintf fmt "Ac:[%a]" AC.print t
-          
-    else 
+    if term_like_pp then 
       match r with
         | X1 t    -> fprintf fmt "%a" X1.print t
         | X2 t    -> fprintf fmt "%a" X2.print t
@@ -130,6 +160,15 @@ struct
         | X5 t    -> fprintf fmt "%a" X5.print t
         | Term t  -> fprintf fmt "%a" Term.print t
         | Ac t    -> fprintf fmt "%a" AC.print t
+    else 
+      match r with
+        | X1 t    -> fprintf fmt "X1(%s):[%a]" X1.name X1.print t
+        | X2 t    -> fprintf fmt "X2(%s):[%a]" X2.name X2.print t
+        | X3 t    -> fprintf fmt "X3(%s):[%a]" X3.name X3.print t
+        | X4 t    -> fprintf fmt "X3(%s):[%a]" X4.name X4.print t
+        | X5 t    -> fprintf fmt "X3(%s):[%a]" X5.name X5.print t
+        | Term t  -> fprintf fmt "FT:[%a]" Term.print t
+        | Ac t    -> fprintf fmt "Ac:[%a]" AC.print t
 
             
   let leaves r = 
@@ -157,7 +196,15 @@ struct
 
   let term_embed t = Term t
 
-  let term_extract = function Term t -> Some t | _ -> None
+  let term_extract r = 
+    match r with 
+      | X1 _ -> X1.term_extract r 
+      | X2 _ -> X2.term_extract r 
+      | X3 _ -> X3.term_extract r 
+      | X4 _ -> X4.term_extract r 
+      | X5 _ -> X5.term_extract r 
+      | Ac _ -> None (* SYLVAIN : TODO *)
+      | Term t -> Some t
 
   let subst p v r = 
     if equal p v then r 
@@ -174,10 +221,10 @@ struct
     let {Term.f=sb} = Term.view t in
     match 
       X1.is_mine_symb sb,
-      X2.is_mine_symb sb,
-      X3.is_mine_symb sb,
-      X4.is_mine_symb sb,
-      X5.is_mine_symb sb,
+      not restricted && X2.is_mine_symb sb,
+      not restricted && X3.is_mine_symb sb,
+      not restricted && X4.is_mine_symb sb,
+      not restricted && X5.is_mine_symb sb,
       AC.is_mine_symb sb 
     with
       | true  , false , false, false, false, false -> X1.make t
@@ -192,10 +239,10 @@ struct
   let fully_interpreted sb =
     match 
       X1.is_mine_symb sb,
-      X2.is_mine_symb sb,
-      X3.is_mine_symb sb,
-      X4.is_mine_symb sb,
-      X5.is_mine_symb sb,
+      not restricted && X2.is_mine_symb sb,
+      not restricted && X3.is_mine_symb sb,
+      not restricted && X4.is_mine_symb sb,
+      not restricted && X5.is_mine_symb sb,
       AC.is_mine_symb sb 
     with
       | true  , false , false, false, false, false -> X1.fully_interpreted sb
@@ -347,15 +394,6 @@ struct
     sbs
 
 
-  let rec type_info = function
-    | X1 t   -> X1.type_info t
-    | X2 t   -> X2.type_info t
-    | X3 t   -> X3.type_info t
-    | X4 t   -> X4.type_info t
-    | X5 t   -> X5.type_info t
-    | Ac x   -> AC.type_info x
-    | Term t -> let {Term.ty = ty} = Term.view t in ty
-	
   module Rel =
   struct
     type elt = r
@@ -377,32 +415,34 @@ struct
       r5=X5.Rel.empty ();
     }
 	
-    let assume env sa expl = 
-      let env1, seq1 = X1.Rel.assume env.r1 sa expl in
-      let env2, seq2 = X2.Rel.assume env.r2 sa expl in
-      let env3, seq3 = X3.Rel.assume env.r3 sa expl in
-      let env4, seq4 = X4.Rel.assume env.r4 sa expl in
-      let env5, seq5 = X5.Rel.assume env.r5 sa expl in
-      {r1=env1; r2=env2; r3=env3; r4=env4; r5=env5}, seq1@seq2@seq3@seq4@seq5
-
-    let instantiate env are_eq are_dist class_of sa = 
-      let env1, seq1 = X1.Rel.instantiate env.r1 are_eq are_dist class_of sa in
-      let env2, seq2 = X2.Rel.instantiate env.r2 are_eq are_dist class_of sa in
-      let env3, seq3 = X3.Rel.instantiate env.r3 are_eq are_dist class_of sa in
-      let env4, seq4 = X4.Rel.instantiate env.r4 are_eq are_dist class_of sa in
-      let env5, seq5 = X5.Rel.instantiate env.r5 are_eq are_dist class_of sa in 
-      {r1=env1; r2=env2; r3=env3; r4=env4; r5=env5}, seq1@seq2@seq3@seq4@seq5
+    let assume env sa ~are_eq ~are_neq ~class_of = 
+      let env1, { assume = a1; remove = rm1} = 
+	X1.Rel.assume env.r1 sa ~are_eq ~are_neq ~class_of in
+      let env2, { assume = a2; remove = rm2} = 
+	X2.Rel.assume env.r2 sa ~are_eq ~are_neq ~class_of in
+      let env3, { assume = a3; remove = rm3} = 
+	X3.Rel.assume env.r3 sa ~are_eq ~are_neq ~class_of in
+      let env4, { assume = a4; remove = rm4} = 
+	X4.Rel.assume env.r4 sa ~are_eq ~are_neq ~class_of in
+      let env5, { assume = a5; remove = rm5} = 
+	X5.Rel.assume env.r5 sa ~are_eq ~are_neq ~class_of in
+      {r1=env1; r2=env2; r3=env3; r4=env4; r5=env5}, 
+      { assume = a1@a2@a3@a4@a5;
+	remove = rm1@rm2@rm3@rm4@rm5;}
 	
-    let query a env expl = 
-      match X1.Rel.query a env.r1 expl with
+    let query env a ~are_eq ~are_neq ~class_of = 
+      match X1.Rel.query env.r1 a ~are_eq ~are_neq ~class_of with
 	| Yes _ as ans -> ans
-	| No -> match X2.Rel.query a env.r2 expl with
+	| No -> 
+	  match X2.Rel.query env.r2 a ~are_eq ~are_neq ~class_of with
 	    | Yes _ as ans -> ans
-	    | No -> match X3.Rel.query a env.r3 expl with
+	    | No ->
+	      match X3.Rel.query env.r3 a ~are_eq ~are_neq ~class_of with
 		| Yes _ as ans -> ans
-		| No -> match X4.Rel.query a env.r4 expl with
-		    | Yes _ as ans -> ans
-		    | No -> X5.Rel.query a env.r5 expl
+		| No -> 
+		    match X4.Rel.query env.r4 a ~are_eq ~are_neq ~class_of with
+		      | Yes _ as ans -> ans
+		      | No -> X5.Rel.query env.r5 a ~are_eq ~are_neq ~class_of
 		      
     let case_split env = 
       let seq1 = X1.Rel.case_split env.r1 in
@@ -434,8 +474,8 @@ and X1 : Sig.THEORY  with type t = TX1.t and type r = CX.r =
        let embed =  CX.embed1
      end)
 
-and X2 : Sig.THEORY with type r = CX.r and type t = CX.r Pairs.abstract =
-  Pairs.Make
+and X2 : Sig.THEORY with type r = CX.r and type t = CX.r Records.abstract =
+  Records.Make
     (struct
        include CX
        let extract = extract2
@@ -458,7 +498,7 @@ and X4 : Sig.THEORY with type r = CX.r and type t = CX.r Arrays.abstract =
        let embed = embed4
      end)
 
- (* Its signature is not Sig.THEORY because it doen't provide a solver *)
+ (* Its signature is not Sig.THEORY because it does not provide a solver *)
 and AC : Ac.S with type r = CX.r = Ac.Make(CX)
 
 and X5 : Sig.THEORY with type r = CX.r and type t = CX.r Sum.abstract =
