@@ -1,12 +1,14 @@
 (**************************************************************************)
 (*                                                                        *)
-(*     The Alt-ergo theorem prover                                        *)
-(*     Copyright (C) 2006-2010                                            *)
+(*     The Alt-Ergo theorem prover                                        *)
+(*     Copyright (C) 2006-2011                                            *)
 (*                                                                        *)
 (*     Sylvain Conchon                                                    *)
 (*     Evelyne Contejean                                                  *)
-(*     Stephane Lescuyer                                                  *)
+(*                                                                        *)
+(*     Francois Bobot                                                     *)
 (*     Mohamed Iguernelala                                                *)
+(*     Stephane Lescuyer                                                  *)
 (*     Alain Mebsout                                                      *)
 (*                                                                        *)
 (*     CNRS - INRIA - Universite Paris Sud                                *)
@@ -58,7 +60,7 @@ let rec apply_subst_formula env f =
 	  | TAlt l -> TAlt (List.map (apply_subst_term env) l)
 	  | TAbuilt(s,l) -> TAbuilt(s,List.map (apply_subst_term env) l)
 	  | TApred t -> TApred (apply_subst_term env t)
-	  | TAfalse | TAtrue -> assert false
+	  | TAfalse | TAtrue -> e.c 
 	in TFatom {e with c = a}
     | TFop (op, lf) ->
 	TFop (op, List.map (apply_subst_formula env) lf)
