@@ -1,12 +1,14 @@
 (**************************************************************************)
 (*                                                                        *)
-(*     The Alt-ergo theorem prover                                        *)
-(*     Copyright (C) 2006-2010                                            *)
+(*     The Alt-Ergo theorem prover                                        *)
+(*     Copyright (C) 2006-2011                                            *)
 (*                                                                        *)
 (*     Sylvain Conchon                                                    *)
 (*     Evelyne Contejean                                                  *)
-(*     Stephane Lescuyer                                                  *)
+(*                                                                        *)
+(*     Francois Bobot                                                     *)
 (*     Mohamed Iguernelala                                                *)
+(*     Stephane Lescuyer                                                  *)
 (*     Alain Mebsout                                                      *)
 (*                                                                        *)
 (*     CNRS - INRIA - Universite Paris Sud                                *)
@@ -43,6 +45,7 @@ module Make
   (P : EXTENDED_Polynome with type r = X.r) = struct
 
   module MP = Map.Make(P)
+  module SP = Set.Make(P)
   module SX = Set.Make(struct type t = X.r include X end)
   module MX = Map.Make(struct type t = X.r include X end)
   
@@ -95,6 +98,7 @@ module Make
     monomes: (Intervals.t * SX.t) MX.t;
     polynomes : Intervals.t MP.t;
     known_eqs : SX.t;
+    improved : SP.t;
   }
 
   module Debug = struct
@@ -157,12 +161,19 @@ module Make
     monomes = MX.empty ; 
     polynomes = MP.empty ; 
     known_eqs = SX.empty ; 
+    improved = SP.empty ; 
   }
 
   let replace_inequation env x ineq = 
     { env with
 	inequations = (x, ineq)::(List.remove_assoc x env.inequations) }
 
+
+  let up_improved env p oldi newi =
+    if Intervals.is_strict_smaller newi oldi then
+      { env with improved = SP.add p env.improved }
+    else env
+    
 (*
   let oldify_inequations env =
     { env with
@@ -310,39 +321,44 @@ module Make
        (*else acc*))
       use_x env' 
 
-  let update_ple0 env p is_le expl =
-    if P.is_empty p then env
-    else 
-      let ty = P.type_info p in
-      let a, _ = P.choose p in
-      let p, change =
-	if a </ Int 0 then
-	  P.mult (P.create [] (Int (-1)) ty) p, true
-	else p, false in
-      let p, c, _ = P.normal_form p in
-      let c = minus_num c in
-      let u =
-	if change then
-	  Intervals.new_borne_inf expl c is_le (Intervals.undefined ty)
-	else
-	  Intervals.new_borne_sup expl c is_le (Intervals.undefined ty) in
-      let u =
-	try 
-	  let pu = MP.find p env.polynomes in
-	  let i = Intervals.intersect u pu in
-	  i
-	with Not_found -> u
-      in
-      { env with polynomes = MP.add p u env.polynomes }
+  let update_monomes_from_poly p i polynomes monomes =
+    let lp, _ = P.to_list p in
+    let ty = P.type_info p in
+    List.fold_left (fun monomes (a,x) ->
+      let np = P.remove x p in
+      let (np,c,d) = P.normal_form_pos np in
+      try 
+	let inp = MP.find np polynomes in
+	(* fprintf fmt "up: %a in %a = (%a in %a, %s, %s) @." *)
+	(*   P.print p Intervals.print i P.print np Intervals.print inp  *)
+	(*   (string_of_num c) (string_of_num d); *)
+	let new_ix =
+	  Intervals.scale 
+	    ((Int 1) // a)
+	    (Intervals.add i
+	       (Intervals.scale (minus_num d)
+		  (Intervals.add inp 
+		     (Intervals.point c ty Explanation.empty)))) in
+	(* fprintf fmt "   %a in %a @." *)
+	(*   X.print x Intervals.print new_ix; *)
+	let old_ix, ux = MX.find x monomes in
+	let ix = Intervals.intersect old_ix new_ix in
+	MX.add x (ix, ux) monomes
+      with Not_found -> monomes)
+      monomes lp
 
   let update_polynomes env expl =
-    let polynomes = MP.fold
-      (fun p i polynomes ->
+    let polynomes, monomes, improved = MP.fold
+      (fun p ip (polynomes, monomes, improved) ->
 	 let new_i = intervals_from_monomes env p in
-	 let i = Intervals.intersect new_i i in
-	 MP.add p i polynomes
-      ) env.polynomes env.polynomes in
-    {env with polynomes = polynomes }
+	 let i = Intervals.intersect new_i ip in
+	 if Intervals.is_strict_smaller i ip then
+	   let monomes = update_monomes_from_poly p i polynomes monomes in
+	   let improved = SP.add p improved in
+	   MP.add p i polynomes, monomes, improved
+	 else polynomes, monomes, improved
+      ) env.polynomes (env.polynomes, env.monomes, env.improved) in
+    {env with polynomes = polynomes; monomes = monomes ; improved = improved}
 
 
   let find_one_eq x u =
@@ -434,6 +450,44 @@ module Make
     let env =  tighten_non_lin x use_x env expl in
     env, (find_eq eqs x u env)
   
+  let update_ple0 env p0 is_le expl =
+    if P.is_empty p0 then env
+    else 
+      let ty = P.type_info p0 in
+      let a, _ = P.choose p0 in
+      let p, change =
+	if a </ Int 0 then
+	  P.mult (P.create [] (Int (-1)) ty) p0, true
+	else p0, false in
+      let p, c, _ = P.normal_form p in
+      let c = minus_num c in
+      let u =
+	if change then
+          Intervals.new_borne_inf expl c is_le (Intervals.undefined ty)
+	else
+	  Intervals.new_borne_sup expl c is_le (Intervals.undefined ty) in
+      let u, pu =
+	try 
+	  let pu = MP.find p env.polynomes in
+	  let i = Intervals.intersect u pu in
+	  i, pu
+	with Not_found -> u, Intervals.undefined ty
+      in
+      let env = 
+	if Intervals.is_strict_smaller u pu then
+	  let polynomes = MP.add p u env.polynomes in
+	  let monomes = update_monomes_from_poly p u polynomes env.monomes in
+	  let improved = SP.add p env.improved in
+	  { env with 
+	    polynomes = polynomes; 
+	    monomes = monomes; 
+	    improved = improved }
+	else env
+      in
+      match P.to_list p0 with
+        | [a,x], v -> fst(update_intervals env [] expl (a, x, v) is_le)
+        | _ -> env
+
   let add_inequations acc lin expl = 
     List.fold_left
       (fun (env, eqs) ineq ->
@@ -581,6 +635,7 @@ module Make
 *)
 
   let fm env eqs expl = 
+    if rules = 4 then fprintf fmt "[rule] TR-Arith-Fm@.";
     fourier (env, eqs) (List.map snd env.inequations) expl
 
   let is_num r = 
@@ -617,7 +672,18 @@ module Make
 	    with Not_found -> Intervals.undefined ty
 	  in
 	  let i = Intervals.exclude i1 i2 in
-	  let env ={ env with polynomes = MP.add p i env.polynomes } in
+	  let env = 
+	    if Intervals.is_strict_smaller i i2 then
+	      let polynomes = MP.add p i env.polynomes in
+	      let monomes = update_monomes_from_poly p i polynomes env.monomes
+	      in
+	      let improved = SP.add p env.improved in
+	      { env with 
+		polynomes = polynomes;
+		monomes = monomes;
+		improved = improved}
+	    else env
+	  in
 	  env, eqs
 					      
   let add_equality env eqs p expl =
@@ -644,16 +710,28 @@ module Make
 	  else P.mult (P.create [] (Int (-1)) ty) p in
 	  let p, c, _ = P.normal_form p in
 	  let i = Intervals.point (minus_num c) ty expl in
-	  let i = 
+	  let i, ip = 
 	    try
-	      let i' =  MP.find p env.polynomes in
-	      Intervals.intersect i i'
-	    with Not_found -> i
+	      let ip =  MP.find p env.polynomes in
+	      Intervals.intersect i ip, ip
+	    with Not_found -> i, Intervals.undefined ty
 	  in
-	  let env = { env with 
-                        polynomes = MP.add p i env.polynomes;
-                        known_eqs = SX.add (P.alien_of p) env.known_eqs
-                    } in
+	  let env = 
+	    if Intervals.is_strict_smaller i ip then
+	      let polynomes = MP.add p i env.polynomes in
+	      let monomes = update_monomes_from_poly p i polynomes env.monomes
+	      in
+	      let improved = SP.add p env.improved in
+	      { env with 
+		polynomes = polynomes;
+		monomes = monomes;
+		improved = improved }
+	    else env
+	  in
+	  let env = 
+	    { env with 
+	      known_eqs = SX.add (P.alien_of p) env.known_eqs
+            } in
 	  env, eqs
 
   let normal_form a = match a with
@@ -716,24 +794,14 @@ module Make
     let env, eqs = equalities_from_polynomes env eqs in
     equalities_from_monomes env eqs
 
-  let assume env la expl = 
+  let assume env la ~are_eq ~are_neq ~class_of =
+    let env = {env with improved = SP.empty} in
     Debug.env env;
-    (*let expl = 
-      List.fold_left 
-      (fun expl (a,_) ->
-      Explanation.union 
-      (Explanation.singleton (Formula.mk_lit a)) expl
-      ) Explanation.empty env.inequations
-      in*)
     let env, eqs, new_ineqs, expl =
       List.fold_left
 	(fun (env, eqs, new_ineqs, expl) (a, root, e) ->
 	   let a = normal_form a in
 	   let expl = Explanation.union e expl in
-	   (*let expl = match root with | Some a ->
-	   (*Explanation.everything*) Explanation.union expl
-	     (Explanation.singleton (Formula.mk_lit a)) | None -> expl
-	     in*)
            if debug_fm then 
 	     begin 
 	       Debug.assume a;
@@ -768,23 +836,15 @@ module Make
 
 	       | _ -> (env, eqs, new_ineqs, expl) 
 		   
-	   with Intervals.NotConsistent expl -> 	       
+	   with Intervals.NotConsistent expl ->
 	     if debug_fm then 
 	       fprintf fmt "interval inconsistent %a@." 
 		 Explanation.print expl; 
 	     raise (Exception.Inconsistent expl)
 	)
-	(env, [], false, expl) la 
+	(env, [], false, Explanation.empty) la 
 	
     in
-    (* explanations for new inequations *)
-    (*let expl = if new_ineqs then 
-      List.fold_left 
-      (fun expl (a,_) ->
-      Explanation.union 
-      (Explanation.singleton (Formula.mk_lit a)) expl
-      ) expl env.inequations
-      else expl in*)
     if new_ineqs then 
       if debug_fm then 
 	fprintf fmt "new explanations %a@." Explanation.print expl; 
@@ -797,17 +857,27 @@ module Make
       Debug.env env;
       let eqs = remove_trivial_eqs eqs la in
       Debug.implied_equalities eqs;
-      env, eqs
+      let result = 
+	List.fold_left 
+	  (fun r (a_sem, a_term, ex) -> 
+	     { assume = (LSem(a_sem), ex) :: r.assume; 
+	       remove = 
+		 match a_term with 
+		   | None -> r.remove 
+		   | Some t -> (LTerm(t), ex)::r.remove
+	     } ) { assume = []; remove = [] } eqs
+      in
+      env, result
 
-    with Intervals.NotConsistent expl -> 
+    with Intervals.NotConsistent expl ->
       if debug_fm then 
 	fprintf fmt "interval inconsistent %a@." 
 	  Explanation.print expl; 
       raise (Exception.Inconsistent expl)
       
-  let query (a,r) env expl =
+  let query env a_ex ~are_eq ~are_neq ~class_of =
     try 
-      ignore(assume env [a,r,Explanation.empty] expl); 
+      ignore(assume env [a_ex] ~are_eq ~are_neq ~class_of); 
       No
     with Exception.Inconsistent expl -> Yes expl
 
@@ -818,18 +888,20 @@ module Make
 	   | Some s when s >/ (Int 1) ->
 	       begin
 		 match o with
-		   | Some (s',p',n') when s' <=/ s -> o
-		   | _ -> Some (s, p, Intervals.borne_inf i)
+		   | Some (s', _, _, _) when s' <=/ s -> o
+		   | _ -> 
+		       let n, ex = Intervals.borne_inf i in
+		       Some (s, p, n, ex)
 	       end
 	   | _ -> o
       ) env.polynomes None in
     match o with 
-      | Some (s,p,n) -> 
+      | Some (s, p, n, ex) -> 
           let r1 = P.alien_of p in
 	  let r2 = P.alien_of (P.create [] n  (P.type_info p)) in
 	  if debug_fm then
 	    fprintf fmt "[case-split] %a = %a@." X.print r1 X.print r2;
-	  [(L.Eq(r1, r2), None), s]
+	  [L.Eq(r1, r2), ex, s]
       | None -> 
 	  if debug_fm then fprintf fmt "[case-split] polynomes: nothing@.";
 	  []
@@ -841,30 +913,37 @@ module Make
 	   | Some s when s >/ (Int 1) ->
 	       begin
 		 match o with
-		   | Some (s',p',n') when s' <=/ s -> o
-		   | _ -> Some (s, x, Intervals.borne_inf i)
+		   | Some (s', _, _, _) when s' <=/ s -> o
+		   | _ -> 
+		       let n, ex = Intervals.borne_inf i in
+		       Some (s, x, n, ex)
 	       end
 	   | _ -> o
       ) env.monomes None in
     match o with 
-      | Some (s,x,n) -> 
+      | Some (s,x,n,ex) -> 
           let ty = X.type_info x in
           let r1 = x in
 	  let r2 = P.alien_of (P.create [] n  ty) in
 	  if debug_fm then
 	    fprintf fmt "[case-split] %a = %a@." X.print r1 X.print r2;
-	  [(L.Eq(r1, r2), None), s]
+	  [L.Eq(r1, r2), ex, s]
       | None -> 
 	  if debug_fm then fprintf fmt "[case-split] monomes: nothing@.";
 	  []
    
   let case_split env = 
+    if rules = 4 then fprintf fmt "[rule] TR-Arith-CaseSplit@.";
     match case_split_polynomes env with
       | []     -> case_split_monomes env
       | choices -> choices
    
   let add env _ = env
 
-  let instantiate env _ _ _ _ = env, []
+  let extract_improved env =
+    SP.fold
+      (fun p acc ->
+	MP.add p (MP.find p env.polynomes) acc)
+      env.improved MP.empty
 
 end
