@@ -1,12 +1,14 @@
 (**************************************************************************)
 (*                                                                        *)
-(*     The Alt-ergo theorem prover                                        *)
-(*     Copyright (C) 2006-2010                                            *)
+(*     The Alt-Ergo theorem prover                                        *)
+(*     Copyright (C) 2006-2011                                            *)
 (*                                                                        *)
 (*     Sylvain Conchon                                                    *)
 (*     Evelyne Contejean                                                  *)
-(*     Stephane Lescuyer                                                  *)
+(*                                                                        *)
+(*     Francois Bobot                                                     *)
 (*     Mohamed Iguernelala                                                *)
+(*     Stephane Lescuyer                                                  *)
 (*     Alain Mebsout                                                      *)
 (*                                                                        *)
 (*     CNRS - INRIA - Universite Paris Sud                                *)
@@ -33,7 +35,7 @@ let print_status d s steps =
   let satmode = !smtfile or !smt2file or !satmode in 
   match s with
     | Unsat dep -> 
-	if not satmode then Loc.report d.st_loc;
+	if not satmode then Loc.report std_formatter d.st_loc;
 	if satmode then printf "@{<C.F_Red>unsat@}@." 
 	else printf "@{<C.F_Green>Valid@} (%2.4f) (%Ld)@." (Time.get()) steps;
 	if proof && not debug_proof then 
@@ -41,17 +43,17 @@ let print_status d s steps =
 	  
     | Inconsistent ->
 	if not satmode then 
-	  (Loc.report d.st_loc; 
+	  (Loc.report std_formatter d.st_loc; 
 	   fprintf fmt "Inconsistent assumption@.")
 	else printf "unsat@."
 	  
     | Unknown ->
 	if not satmode then
-	  (Loc.report d.st_loc; printf "I don't know.@.")
+	  (Loc.report std_formatter d.st_loc; printf "I don't know.@.")
 	else printf "unknown@."
 	  
     | Sat  -> 
-	if not satmode then Loc.report d.st_loc;
+	if not satmode then Loc.report std_formatter d.st_loc;
 	if satmode then printf "unknown (sat)@." 
 	else printf "I don't know@."
 
@@ -64,17 +66,17 @@ let main _ =
     processing print_status d
   with
     | Why_lexer.Lexical_error s -> 
-	Loc.report (lexeme_start_p lb, lexeme_end_p lb);
-	printf "lexical error: %s\n@." s;
+	Loc.report err_formatter (lexeme_start_p lb, lexeme_end_p lb);
+	eprintf "lexical error: %s\n@." s;
 	exit 1
     | Parsing.Parse_error ->
 	let  loc = (lexeme_start_p lb, lexeme_end_p lb) in
-	Loc.report loc;
-        printf "syntax error\n@.";
+	Loc.report err_formatter loc;
+        eprintf "syntax error\n@.";
 	exit 1
     | Common.Error(e,l) -> 
-	Loc.report l; 
-	printf "typing error: %a\n@." Common.report e;
+	Loc.report err_formatter l; 
+	eprintf "typing error: %a\n@." Common.report e;
 	exit 1
 
 let _ = main ();
