@@ -1,12 +1,14 @@
 (**************************************************************************)
 (*                                                                        *)
-(*     The Alt-ergo theorem prover                                        *)
-(*     Copyright (C) 2006-2010                                            *)
+(*     The Alt-Ergo theorem prover                                        *)
+(*     Copyright (C) 2006-2011                                            *)
 (*                                                                        *)
 (*     Sylvain Conchon                                                    *)
 (*     Evelyne Contejean                                                  *)
-(*     Stephane Lescuyer                                                  *)
+(*                                                                        *)
+(*     Francois Bobot                                                     *)
 (*     Mohamed Iguernelala                                                *)
+(*     Stephane Lescuyer                                                  *)
 (*     Alain Mebsout                                                      *)
 (*                                                                        *)
 (*     CNRS - INRIA - Universite Paris Sud                                *)
@@ -17,11 +19,12 @@
 
 type gsubst = { sbt : Term.subst ; gen : int ; goal : bool}
 
-type pat_info = {
-  pat_age : int ; 
-  pat_orig : Formula.t ; 
-  pat_formula : Formula.t ; 
-  pat_dep : Explanation.t ;
+type trigger_info = {
+  trigger_query : Literal.LT.t option; 
+  trigger_age : int ; 
+  trigger_orig : Formula.t ; 
+  trigger_formula : Formula.t ; 
+  trigger_dep : Explanation.t ;
 }
 
 type term_info = {
@@ -43,8 +46,8 @@ module type S = sig
 
   val empty : t
   val add_term : term_info -> Term.t -> t -> t 
-  val add_pat : pat_info * Term.t list -> t -> uf -> t
-  val query : t -> uf -> (pat_info * gsubst list) list
+  val add_trigger : trigger_info -> Term.t list -> t -> t
+  val query : t -> uf -> (trigger_info * gsubst list) list
 
 end
 
