@@ -1,12 +1,14 @@
 (**************************************************************************)
 (*                                                                        *)
-(*     The Alt-ergo theorem prover                                        *)
-(*     Copyright (C) 2006-2010                                            *)
+(*     The Alt-Ergo theorem prover                                        *)
+(*     Copyright (C) 2006-2011                                            *)
 (*                                                                        *)
 (*     Sylvain Conchon                                                    *)
 (*     Evelyne Contejean                                                  *)
-(*     Stephane Lescuyer                                                  *)
+(*                                                                        *)
+(*     Francois Bobot                                                     *)
 (*     Mohamed Iguernelala                                                *)
+(*     Stephane Lescuyer                                                  *)
 (*     Alain Mebsout                                                      *)
 (*                                                                        *)
 (*     CNRS - INRIA - Universite Paris Sud                                *)
@@ -17,6 +19,7 @@
 
 open Format
 open Num
+open Options
 
 exception Not_a_num
 exception Maybe_zero
@@ -66,16 +69,43 @@ module Make (X : S) = struct
   type r = X.r
       
   module M : Map.S with type key = r = 
-    Map.Make(struct type t = r let compare x y = X.compare y x end)
+    Map.Make(
+      struct 
+        type t = r 
+            
+        (*sorted in decreasing order to comply with AC(X) order requirements*)
+        let compare x y = X.compare y x
+      end)
       
   type t = { m : num M.t; c : num; ty : Ty.t }
 
+
+  let map_to_list m = List.rev (M.fold (fun x a aliens -> (a, x)::aliens) m [])
+
+  exception Out of int
+
+  let compare_maps l1 l2 = 
+    try
+      List.iter2
+        (fun (a,x) (b,y) ->
+           let c = X.compare x y   in if c <> 0 then raise (Out c);
+           let c = compare_num a b in if c <> 0 then raise (Out c) )l1 l2;
+      0
+    with
+      | Out c -> c 
+      | Invalid_argument("List.iter2") -> List.length l1 - List.length l2
+            
   let compare p1 p2 = 
     let c = Ty.compare p1.ty p2.ty in
     if c <> 0 then c
-    else
-      let c = compare_num p1.c p2.c in
-      if c = 0 then M.compare compare_num p1.m p2.m else c
+    else match M.is_empty p1.m, M.is_empty p2.m with
+      | true , false -> -1
+      | false, true  -> 1
+      | true , true  -> compare_num p1.c p2.c
+      | false, false ->
+          let c =  compare_maps (map_to_list p1.m) (map_to_list p2.m) in
+          if c = 0 then compare_num p1.c p2.c else c
+
 
   let hash p = 
     abs (Hashtbl.hash p.m + 19*Hashtbl.hash p.c + 17 * Ty.hash p.ty)
@@ -122,6 +152,7 @@ module Make (X : S) = struct
     { m = m; c = c; ty = ty }
       
   let add p1 p2 = 
+    if rules = 4 then fprintf fmt "[rule] TR-Arith-Poly plus@.";
     let m = 
       M.fold 
 	(fun x a m -> 
@@ -145,13 +176,16 @@ module Make (X : S) = struct
     { acx with m = m}
       
   let mult p1 p2 =
+    if rules = 4 then fprintf fmt "[rule] TR-Arith-Poly mult@.";
     let p = mult_const p1.c p2 in
     M.fold (fun x a p -> add (mult_monome a x p2) p) p1.m p
 
-  let sub p1 p2 = 
+  let sub p1 p2 =
+    if rules = 4 then fprintf fmt "[rule] TR-Arith-Poly moins@."; 
     add p1 (mult (create [] (Int (-1)) p1.ty) p2)
 
-  let div p1 p2 = 
+  let div p1 p2 =
+    if rules = 4 then fprintf fmt "[rule] TR-Arith-Poly div@.";
     if M.is_empty p2.m then
       if p2.c =/ Int 0 then raise Division_by_zero
       else 
@@ -165,7 +199,8 @@ module Make (X : S) = struct
     else raise Maybe_zero
 
 
-  let modulo p1 p2 = 
+  let modulo p1 p2 =
+    if rules = 4 then fprintf fmt "[rule] TR-Arith-Poly mod@.";
     if M.is_empty p2.m then
       if p2.c =/ Int 0 then raise Division_by_zero
       else 
@@ -194,9 +229,7 @@ module Make (X : S) = struct
       
   let remove x p = { p with m = M.remove x p.m }
       
-  let to_list p = 
-    let l = M.fold (fun x a aliens -> (a, x)::aliens ) p.m [] in
-    List.rev l, p.c
+  let to_list p = map_to_list p.m , p.c
 
   let type_info p = p.ty
 
