@@ -1,12 +1,14 @@
 (**************************************************************************)
 (*                                                                        *)
-(*     The Alt-ergo theorem prover                                        *)
-(*     Copyright (C) 2006-2010                                            *)
+(*     The Alt-Ergo theorem prover                                        *)
+(*     Copyright (C) 2006-2011                                            *)
 (*                                                                        *)
 (*     Sylvain Conchon                                                    *)
 (*     Evelyne Contejean                                                  *)
-(*     Stephane Lescuyer                                                  *)
+(*                                                                        *)
+(*     Francois Bobot                                                     *)
 (*     Mohamed Iguernelala                                                *)
+(*     Stephane Lescuyer                                                  *)
 (*     Alain Mebsout                                                      *)
 (*                                                                        *)
 (*     CNRS - INRIA - Universite Paris Sud                                *)
@@ -19,9 +21,15 @@ type answer = Yes of Explanation.t | No
 
 type 'a ac = {h: Symbols.t ; t: Ty.t ; l: ('a * int) list}
 
-type 'a sem_atom =  'a Literal.view * Literal.LT.t option
+type 'a literal = LSem of 'a Literal.view | LTerm of Literal.LT.t
 
-type 'a sem_atom_ex =  'a Literal.view * Literal.LT.t option * Explanation.t
+type 'a input =  
+    'a Literal.view * Literal.LT.t option * Explanation.t
+
+type 'a result = { 
+  assume : ('a literal * Explanation.t) list;  
+  remove: ('a literal * Explanation.t) list;
+}
 
 module type RELATION = sig
   type t
@@ -30,19 +38,22 @@ module type RELATION = sig
   val empty : unit -> t
   
   val assume : 
-    t -> r sem_atom_ex list -> Explanation.t -> t * r sem_atom_ex list
-
-  val instantiate :
     t -> 
-    (Term.t -> Term.t -> answer) -> 
-    (Term.t -> Term.t -> answer) -> 
-    (Term.t -> Term.t list)      ->
-    r sem_atom_ex list ->
-    t * (Literal.LT.t * Explanation.t) list
+    (r input) list -> 
+    are_eq : (Term.t -> Term.t -> answer) -> 
+    are_neq : (Term.t -> Term.t -> answer) -> 
+    class_of : (Term.t -> Term.t list) -> 
+    t * r result
 
-  val query : r sem_atom -> t -> Explanation.t -> answer
+  val query : 
+    t -> 
+    r input -> 
+    are_eq : (Term.t -> Term.t -> answer) -> 
+    are_neq : (Term.t -> Term.t -> answer) -> 
+    class_of : (Term.t -> Term.t list) -> 
+    answer
 
-  val case_split : t -> (r sem_atom * Num.num) list
+  val case_split : t -> (r Literal.view * Explanation.t * Num.num) list
     (** case_split env returns a list of equalities *)
     
   val add : t -> r -> t
@@ -59,11 +70,11 @@ module type THEORY = sig
   (** Name of the theory*)
   val name : string
   (** return true if the atom is owned by the theory*)
-  val is_mine_a : Literal.LT.t -> bool
+  (*val is_mine_a : Literal.LT.t -> bool*)
   (** return true if the symbol is owned by the theory*)
   val is_mine_symb : Symbols.t -> bool
   (** return true if the type is owned by the theory*)
-  val is_mine_type : t -> bool
+  (*val is_mine_type : t -> bool*)
 
   (** return true when the argument is an unsolvable function of the theory *)
   val unsolvable : t -> bool
@@ -71,6 +82,8 @@ module type THEORY = sig
   (** Give a representant of a term of the theory*)
   val make : Term.t -> r * Literal.LT.t list
 
+  val term_extract : r -> Term.t option
+
   val color : (r ac) -> r
   
   val type_info : t -> Ty.t
@@ -81,7 +94,7 @@ module type THEORY = sig
   val leaves : t -> r list
   val subst : r -> r -> t -> r
 
-  val compare : t -> t -> int
+  val compare : r -> r -> int
 
   val hash : t -> int
   (** solve r1 r2, solve the equality r1=r2 and return the substitution *)
