@@ -1,12 +1,14 @@
 (**************************************************************************)
 (*                                                                        *)
-(*     The Alt-ergo theorem prover                                        *)
-(*     Copyright (C) 2006-2010                                            *)
+(*     The Alt-Ergo theorem prover                                        *)
+(*     Copyright (C) 2006-2011                                            *)
 (*                                                                        *)
 (*     Sylvain Conchon                                                    *)
 (*     Evelyne Contejean                                                  *)
-(*     Stephane Lescuyer                                                  *)
+(*                                                                        *)
+(*     Francois Bobot                                                     *)
 (*     Mohamed Iguernelala                                                *)
+(*     Stephane Lescuyer                                                  *)
 (*     Alain Mebsout                                                      *)
 (*                                                                        *)
 (*     CNRS - INRIA - Universite Paris Sud                                *)
@@ -16,7 +18,8 @@
 (**************************************************************************)
 
 type operator = 
-    Plus | Minus | Mult | Div | Modulo | Concat | Extract | Get | Set
+  | Plus | Minus | Mult | Div | Modulo | Concat | Extract 
+  | Get | Set | Access of Hstring.t | Record
 
 type name_kind = Ac | Constructor | Other
 
