@@ -1,12 +1,14 @@
 (**************************************************************************)
 (*                                                                        *)
-(*     The Alt-ergo theorem prover                                        *)
-(*     Copyright (C) 2006-2010                                            *)
+(*     The Alt-Ergo theorem prover                                        *)
+(*     Copyright (C) 2006-2011                                            *)
 (*                                                                        *)
 (*     Sylvain Conchon                                                    *)
 (*     Evelyne Contejean                                                  *)
-(*     Stephane Lescuyer                                                  *)
+(*                                                                        *)
+(*     Francois Bobot                                                     *)
 (*     Mohamed Iguernelala                                                *)
+(*     Stephane Lescuyer                                                  *)
 (*     Alain Mebsout                                                      *)
 (*                                                                        *)
 (*     CNRS - INRIA - Universite Paris Sud                                *)
@@ -20,6 +22,8 @@ open Options
 open Format
 open Why_ptree
 
+type polarity = Pos | Neg
+
 module Vterm = Symbols.Set
 module Vtype = Set.Make(struct type t=int let compare=Pervasives.compare end)
 
@@ -37,6 +41,7 @@ module STRS = Set.Make(
 	  if c=0 then 
 	    let c=compare_term a1 a2 in if c=0 then compare_term b1 b2 else c
 	  else c
+      | TTconst (Treal r1) , TTconst (Treal r2) -> Num.compare_num r1 r2
       | x , y -> Pervasives.compare x y
     and compare_list l1 l2 = match l1,l2 with
 	[] , _ -> -1
@@ -47,8 +52,9 @@ module STRS = Set.Make(
     let compare (t1,_,_) (t2,_,_) = compare_term t1 t2
   end)
 
-let sort = 
-  List.sort (fun l1 l2 -> compare (List.length l1) (List.length l2) )
+let sort = List.sort (fun l1 l2 -> compare (List.length l1) (List.length l2))
+
+let neg_pol x = x (*function Pos -> Neg | Neg -> Pos*)
       
 let compare_tconstant c1 c2 =
   match c1, c2 with
@@ -73,6 +79,10 @@ let rec depth_tterm t =
 	0 (* arithmetic triggers are not suitable *)
     | TTget (t1, t2) | TTconcat (t1, t2) ->
 	max (depth_tterm t1) (depth_tterm t2)
+    | TTdot(t, _) -> 1 + depth_tterm t
+    | TTrecord lbs ->
+	1 + (List.fold_left 
+	       (fun acc (lb, t) -> max (depth_tterm t) acc) 0 lbs)
     | TTset (t1, t2, t3) | TTextract (t1, t2, t3) ->
 	max (depth_tterm t1) (max (depth_tterm t2) (depth_tterm t3))
     | TTlet (s, t1, t2) ->
@@ -153,6 +163,32 @@ let rec compare_tterm t1 t2 =
 	if c<>0 then c else compare_tterm t2 u2
     | TTconcat _, _ -> -1
     | _, TTconcat _ -> 1
+    | TTdot(t1, a1), TTdot(t2,a2) ->
+	let c = compare a1 a2 in
+	if c<>0 then c else
+	  compare_tterm t1 t2
+    | TTdot _, _ -> -1
+    | _, TTdot _ -> 1
+    | TTrecord lbs1, TTrecord lbs2 ->
+	let s1 = List.length lbs1 in
+	let s2 = List.length lbs2 in
+	let c = compare s1 s2 in
+	if c <> 0 then c
+	else
+	  begin
+	    try
+	      List.iter2 
+		(fun (lb1, t1) (lb2, t2) -> 
+		   let c = Hstring.compare lb1 lb2 in
+		   if c<>0 then raise (Out c);
+		   let c = compare_tterm t1 t2 in
+		   if c<>0 then raise (Out c)) 
+		lbs1 lbs2;
+	      0
+	    with Out n -> n
+	  end
+    | TTrecord _, _ -> -1
+    | _, TTrecord _ -> 1
     | TTlet (s1, t1, u1) , TTlet (s2, t2, u2) ->
 	let c = Symbols.compare s1 s2 in
 	if c<>0 then c else
@@ -203,35 +239,33 @@ module SLLT =
   Set.Make(
     struct 
       type t = (int tterm, int) annoted list * Vterm.t * Vtype.t
-      let compare (_, y1, _) (_, y2, _)  = 
-	Vterm.compare y1 y2
+      let compare (_, y1, _) (_, y2, _)  = Vterm.compare y1 y2
     end)
 
 let parties bv vty l = 
   let rec parties_rec (llt, llt_ok)  l = 
     match l with 
-	[] -> llt_ok
+      | [] -> llt_ok
       | (t, bv1, vty1)::l -> 
 	  let llt, llt_ok = 
 	    SLLT.fold
 	      (fun (l, bv2, vty2) (llt, llt_ok) ->
-		 let lt, bv3, vty3 = 
-		   t::l, Vterm.union bv2 bv1, Vtype.union vty2 vty1 
-		 in 
-		 let e = (lt, bv3, vty3) in
+		 let bv3 = Vterm.union bv2 bv1 in
+		 let vty3 = Vtype.union vty2 vty1 in
+		 let e = t::l, bv3, vty3 in
 		 if Vterm.subset bv bv3 && Vtype.subset vty vty3 then
 		   llt, SLLT.add e llt_ok
 		 else 
 		   SLLT.add e llt, llt_ok) 
 	      llt (llt, llt_ok)
 	  in
-	  parties_rec (SLLT.add ([t],bv1,vty1) llt, llt_ok) l
+	  parties_rec (SLLT.add ([t], bv1, vty1) llt, llt_ok) l
   in 
   SLLT.elements (parties_rec (SLLT.empty, SLLT.empty) l)
 	 
 let strict_subset bv vty = 
   List.exists 
-    (fun (_,bv',vty') -> 
+    (fun (_, bv',vty') -> 
        (Vterm.subset bv bv' && not(Vterm.equal bv bv')  
 	&& Vtype.subset vty vty') 
        or (Vtype.subset vty vty' && not(Vtype.equal vty vty') 
@@ -256,6 +290,9 @@ let rec vars_of_term bv acc t = match t.c.tt_desc with
   | TTset (t1, t2, t3) -> List.fold_left (vars_of_term bv) acc [t1;t2;t3]
   | TTget (t1, t2) -> List.fold_left (vars_of_term bv) acc [t1;t2]
   | TTlet (_, t1, t2) -> List.fold_left (vars_of_term bv) acc [t1;t2]
+  | TTdot (t1, _) -> vars_of_term bv acc t1
+  | TTrecord lbs -> 
+      List.fold_left (fun acc (_, t) -> vars_of_term bv acc t) acc lbs
   (* XXX TTlet ? *)
   | _ -> acc
 
@@ -279,7 +316,7 @@ let underscoring_term mvars underscores t =
 let underscoring_mt bv mt = 
   let vars , mvars = 
     List.fold_left 
-      (fun (vars,mvars) t -> 
+      (fun (vars, mvars) t -> 
 	 let vs = vars_of_term bv Vterm.empty t in
 	 let mvars = Vterm.union mvars (Vterm.inter vars vs) in
 	 Vterm.union vars vs , mvars) (Vterm.empty,Vterm.empty) mt in
@@ -289,25 +326,22 @@ let underscoring_mt bv mt =
 let multi_triggers gopt bv vty trs =
   let terms = simplification bv vty trs in
   let l_parties = parties bv vty terms  in 
-  let lm = List.map (fun (lt,_,_) -> lt) l_parties in
-(*    List.fold_left 
-      (fun acc (lt, bv', vty') ->
-	 if Vterm.subset bv bv' && Vtype.subset vty vty' then lt::acc else acc) 
-      [] l_parties*)
+  let lm = List.map (fun (lt, _, _) -> lt) l_parties in
   let mv , mt = List.partition (List.exists is_var) lm in
   let mv , mt = sort mv , sort mt in
   let lm = if gopt || triggers_var then mt@mv else mt in
   let m = at_most redondance  lm in
-  (*(List.map (underscoring_mt bv) m)@
-    (List.map (underscoring_mt bv) (List.map List.rev m))*)
-  at_most redondance m 
+  at_most redondance m
 
 let rec vty_ty acc t = 
   let t = Ty.shorten t in
   match t with
-    | Ty.Tvar { Ty.v = i ; value = None } -> Vtype.add i acc 
+    | Ty.Tvar { Ty.v = i; value = None } -> Vtype.add i acc 
     | Ty.Text(l,_) -> List.fold_left vty_ty acc l
     | Ty.Tfarray (t1,t2) -> vty_ty (vty_ty acc t1) t2
+    | Ty.Trecord {Ty.args = args; lbs = lbs} ->
+	let acc = List.fold_left vty_ty acc args in
+	List.fold_left (fun acc (_, t) -> vty_ty acc t) acc lbs
     | _ -> acc
     
 let rec vty_term acc t = 
@@ -317,6 +351,8 @@ let rec vty_term acc t =
     | TTinfix (t1,_,t2) -> vty_term (vty_term acc t1) t2
     | TTset (t1, t2, t3) -> List.fold_left vty_term acc [t1;t2;t3]
     | TTget (t1, t2) -> List.fold_left vty_term acc [t1;t2]
+    | TTdot (t1, _) -> vty_term acc t1
+    | TTrecord lbs -> List.fold_left (fun acc (_, t) -> vty_term acc t) acc lbs
     | TTlet (_, t1, t2) -> List.fold_left vty_term acc [t1;t2]
     | _ -> acc
 
@@ -340,42 +376,61 @@ let csort = Symbols.name "c_sort"
 let filter_mono vterm vtype (t, bv_t, vty_t) = 
   Vterm.subset vterm bv_t && Vtype.subset vtype vty_t && 
     match t.c.tt_desc with
-      | TTapp(s,_) -> 
-	  not (Symbols.equal s csort)
+      | TTapp(s, _) -> not (Symbols.equal s csort)
       | _ -> true
-	    
-
+	  
 let as_bv bv s = not (Vterm.is_empty (Vterm.inter bv s))
 let as_tyv vty s = not (Vtype.is_empty (Vtype.inter vty s))
 
 let potential_triggers = 
-  let rec potential_rec ((bv,vty) as vars) acc t = 
+  let rec potential_rec ( (bv, vty) as vars) acc t = 
     let vty_t = vty_term Vtype.empty t in
     match t.c.tt_desc with
       | TTvar x ->
 	  if Vterm.mem x bv or as_tyv vty vty_t then
-	    STRS.add (t,Vterm.singleton x, vty_t) acc
+	    STRS.add (t, Vterm.singleton x, vty_t) acc
 	  else acc
+
+      | TTapp(s, lf) when 
+	  Symbols.equal s Common.fake_eq 
+	  or Symbols.equal s Common.fake_neq 
+	  or Symbols.equal s Common.fake_lt 
+	  or Symbols.equal s Common.fake_le -> 
+
+          let vty_lf = List.fold_left vty_term vty_t lf in
+	  let bv_lf = List.fold_left (vars_of_term bv) Vterm.empty lf in
+	  if as_bv bv bv_lf or as_tyv vty vty_lf then
+	    let csts = 
+	      List.filter 
+		(fun f -> 
+		   not (as_bv bv (vars_of_term bv Vterm.empty f)) && 
+		     not (as_tyv vty (vty_term vty f))) lf in
+	    let lf' =  lf@csts in
+	    let t = { t with c = {t.c with tt_desc = TTapp(s, lf')}} in
+	    List.fold_left (potential_rec vars) 
+	      (STRS.add (t, bv_lf, vty_lf) acc) lf
+	  else acc
+
       | TTapp(s,lf)-> 
 	  let vty_lf = List.fold_left vty_term vty_t lf in
 	  let bv_lf = List.fold_left (vars_of_term bv) Vterm.empty lf in
 	  if as_bv bv bv_lf or as_tyv vty vty_lf then
 	    List.fold_left (potential_rec vars) 
-	      (STRS.add (t,bv_lf,vty_lf) acc) lf
+	      (STRS.add (t, bv_lf, vty_lf) acc) lf
 	  else acc
       | TTinfix(t1,_,t2) | TTlet (_, t1, t2) -> (* XXX TTlet ? *)
 	  let vty_lf = List.fold_left vty_term vty_t [t1;t2] in
 	  let bv_lf = List.fold_left (vars_of_term bv) Vterm.empty [t1;t2] in
 	  if as_bv bv bv_lf or as_tyv vty vty_lf then
-	    List.fold_left (potential_rec vars) 
-	      (STRS.add (t,bv_lf,vty_lf) acc) (*acc*) [t1;t2]
+	    List.fold_left 
+	      (potential_rec vars) (STRS.add (t, bv_lf, vty_lf) acc) [t1;t2]
 	  else acc
       | TTset (t1, t2, t3) ->
 	  let vty_lf = List.fold_left vty_term vty_t [t1;t2;t3] in
 	  let bv_lf = List.fold_left (vars_of_term bv) Vterm.empty [t1;t2;t3] in
 	  if as_bv bv bv_lf or as_tyv vty vty_lf then
 	    List.fold_left (potential_rec vars) 
-	      (STRS.add (t,bv_lf,vty_lf) acc) (*acc*) [t1;t2;t3]
+	      (STRS.add (t, bv_lf, vty_lf) acc) [t1;t2;t3]
 	  else acc
 	    
       | TTget (t1, t2) ->
@@ -383,14 +438,29 @@ let potential_triggers =
 	  let bv_lf = List.fold_left (vars_of_term bv) Vterm.empty [t1;t2] in
 	  if as_bv bv bv_lf or as_tyv vty vty_lf then
 	    List.fold_left (potential_rec vars) 
-	      (STRS.add (t,bv_lf,vty_lf) acc) (*acc*) [t1;t2]
+	      (STRS.add (t, bv_lf, vty_lf) acc) [t1;t2]
+	  else acc
+      | TTdot (t1 , a) -> 
+	  let vty_lf = vty_term vty_t t1 in
+	  let bv_lf = vars_of_term bv Vterm.empty t1 in
+	  if as_bv bv bv_lf or as_tyv vty vty_lf then
+	      potential_rec vars (STRS.add (t, bv_lf, vty_lf) acc) t1
+	  else acc
+	  
+      | TTrecord lbs ->
+	  let lt = List.map snd lbs in
+	  let vty_lf = List.fold_left vty_term vty_t lt in
+	  let bv_lf = List.fold_left (vars_of_term bv) Vterm.empty lt in
+	  if as_bv bv bv_lf or as_tyv vty vty_lf then
+	    List.fold_left (potential_rec vars) 
+	      (STRS.add (t, bv_lf, vty_lf) acc) lt
 	  else acc
 
       | _ -> acc
 
   in fun vars -> List.fold_left (potential_rec vars) STRS.empty
 
-let filter_good_triggers (bv,vty) = 
+let filter_good_triggers (bv, vty) = 
   List.filter 
     (fun l ->
        let s1 = List.fold_left (vars_of_term bv) Vterm.empty l in
@@ -402,123 +472,159 @@ let make_triggers gopt vterm vtype trs =
       [] -> 
 	multi_triggers gopt vterm vtype trs
     | trs' -> 
-	let f l = at_most redondance (List.map (fun (t,_,_) -> [t]) l) in
-	let trs_v, trs_nv = 
-	  List.partition (fun (t,_,_) -> is_var t) trs' in
+	let f l = at_most redondance (List.map (fun (t, _, _) -> [t]) l) in
+	let trs_v, trs_nv = List.partition (fun (t, _, _) -> is_var t) trs' in
 	let ll = 
-	  if trs_nv=[] then
+	  if trs_nv = [] then
 	    if triggers_var || gopt then 
 	      f trs_v 
-	    else [](*multi_triggers vars trs*)
+	    else [] (*multi_triggers vars trs*)
 	  else f trs_nv 
 	in 
 	if glouton then ll@(multi_triggers gopt vterm vtype trs) else ll
 
-let rec make_rec gopt vterm vtype f = 
+let rec make_rec pol gopt vterm vtype f = 
   let c, trs = match f.c with
-    | TFatom {c=(TAfalse | TAtrue)} ->
-      f.c, STRS.empty
-    | TFatom a ->
-      if Vterm.is_empty vterm && Vtype.is_empty vtype then 
+    | TFatom {c = (TAfalse | TAtrue)} ->
 	f.c, STRS.empty
-      else 
-	let l = match a.c with    
-	  | TAeq l | TAneq l | TAle l | TAlt l | TAbuilt(_,l) -> l
-	  | TApred t -> [t]
-	  | _ -> assert false
-	in
-	f.c, potential_triggers (vterm, vtype) l
-    | TFop(op, lf) -> 
-      let lf, trs = 
-	List.fold_left
-	  (fun (lf, trs1) f ->
-	    let f, trs2 = make_rec gopt vterm vtype f in
-	    f::lf, STRS.union trs1 trs2) ([], STRS.empty) lf in
-      TFop(op,List.rev lf), trs
+    | TFatom a ->
+	if Vterm.is_empty vterm && Vtype.is_empty vtype then 
+	    f.c, STRS.empty
+	else 
+	  begin
+	  let l = match a.c with    
+	    | TAeq l when pol = Neg -> 
+		let v = 
+		  {tt_desc = TTapp(Common.fake_eq, l); tt_ty = Ty.Tbool} 
+		in
+		[ { c = v; annot = a.annot } ]
+	    | TAneq ([t1; t2] as l) when pol = Neg -> 
+		let v =
+		  { tt_desc = TTapp(Common.fake_neq, l); tt_ty = Ty.Tbool} 
+		in
+		[ { c = v; annot = a.annot } ]
+	    | TAle l when pol = Neg -> 
+		let v =
+		  { tt_desc = TTapp(Common.fake_le, l); tt_ty = Ty.Tbool} 
+		in
+		[ { c = v; annot = a.annot } ]
+	    | TAlt l when pol = Neg ->
+		let v =
+		  { tt_desc = TTapp(Common.fake_lt, l); tt_ty = Ty.Tbool} 
+		in
+		[ { c = v; annot = a.annot } ]
+	    | TAle l | TAlt l | TAeq l | TAneq l | TAbuilt(_,l) -> l
+	    | TApred t -> [t]
+	    | _ -> assert false
+	  in
+	  f.c, potential_triggers (vterm, vtype) l
+	  end
+    | TFop (OPimp, [f1; f2]) -> 
+	
+	let f1, trs1  = make_rec (neg_pol pol) gopt vterm vtype f1 in
+	let f2, trs2  = make_rec pol gopt vterm vtype f2 in
+	let trs = STRS.union trs1 trs2 in
+	TFop(OPimp, [f1; f2]), trs
+
+    | TFop (OPnot, [f1]) -> 
+	let f1, trs1  = make_rec (neg_pol pol) gopt vterm vtype f1 in
+	TFop(OPnot, [f1]), trs1
+
+    (* | OPiff 
+       | OPif of ('a tterm, 'a) annoted *)
+
+    | TFop (op, lf) -> 
+	let lf, trs = 
+	  List.fold_left
+	    (fun (lf, trs1) f ->
+	       let f, trs2 = make_rec pol gopt vterm vtype f in
+	       f::lf, STRS.union trs1 trs2) ([], STRS.empty) lf in
+	TFop(op,List.rev lf), trs
 
     | TFforall ({ qf_form= {c = TFop(OPiff,[{c=TFatom _} as f1;f2]); 
 			    annot = ido}} as qf) -> 
-      let vtype' = vty_form Vtype.empty qf.qf_form in
-      let vterm' = 
-	List.fold_left (fun b (s,_) -> Vterm.add s b) Vterm.empty qf.qf_bvars 
-      in
+	let vtype' = vty_form Vtype.empty qf.qf_form in
+	let vterm' = 
+	  List.fold_left (fun b (s,_) -> Vterm.add s b) Vterm.empty qf.qf_bvars 
+	in
 
-      let vterm'' = Vterm.union vterm vterm' in
-      let vtype'' = Vtype.union vtype vtype' in
-      let f1', trs1 = make_rec gopt vterm'' vtype'' f1 in
-      let f2', trs2 = make_rec gopt vterm'' vtype'' f2 in
-      let trs12 = 
-	if Options.notriggers || qf.qf_triggers = [] then
-	  begin
-	    (make_triggers false vterm' vtype' (STRS.elements trs1))@
-	      (make_triggers false vterm' vtype' (STRS.elements trs2))
-	  end
-	else 
-	  begin
-	    let lf = filter_good_triggers (vterm', vtype') qf.qf_triggers in
-	    if lf<>[] then lf
-	    else
+	let vterm'' = Vterm.union vterm vterm' in
+	let vtype'' = Vtype.union vtype vtype' in
+	let f1', trs1 = make_rec pol gopt vterm'' vtype'' f1 in
+	let f2', trs2 = make_rec pol gopt vterm'' vtype'' f2 in
+	let trs12 = 
+	  if Options.notriggers || qf.qf_triggers = [] then
+	    begin
 	      (make_triggers false vterm' vtype' (STRS.elements trs1))@
 		(make_triggers false vterm' vtype' (STRS.elements trs2))
-	  end
-      in
-      let trs = 
-	STRS.filter 
-	  (fun (_, bvt, _) -> Vterm.is_empty (Vterm.inter bvt vterm')) 
-	  (STRS.union trs1 trs2) in
-      let r  = 
-	{ qf with 
-	  qf_triggers = trs12 ; 
-	  qf_form = {c=TFop(OPiff,[f1'; f2']); annot = ido} }
-      in
-      begin
-	match f.c with 
-	  | TFforall _ -> TFforall r, trs 
-	  | _ -> TFexists r , trs
-      end
+	    end
+	  else 
+	    begin
+	      let lf = filter_good_triggers (vterm', vtype') qf.qf_triggers in
+	      if lf<>[] then lf
+	      else
+		(make_triggers false vterm' vtype' (STRS.elements trs1))@
+		  (make_triggers false vterm' vtype' (STRS.elements trs2))
+	    end
+	in
+	let trs = 
+	  STRS.filter 
+	    (fun (_, bvt, _) -> Vterm.is_empty (Vterm.inter bvt vterm')) 
+	    (STRS.union trs1 trs2) in
+	let r  = 
+	  { qf with 
+	      qf_triggers = trs12 ; 
+	      qf_form = {c=TFop(OPiff,[f1'; f2']); annot = ido} }
+	in
+	begin
+	  match f.c with 
+	    | TFforall _ -> TFforall r, trs 
+	    | _ -> TFexists r , trs
+	end
 
-  | TFforall qf | TFexists qf -> 
-      let vtype' = vty_form Vtype.empty qf.qf_form in
-      let vterm' = 
-	List.fold_left (fun b (s,_) -> Vterm.add s b) Vterm.empty qf.qf_bvars in
-      let f', trs = 
-	make_rec gopt 
-	  (Vterm.union vterm vterm') (Vtype.union vtype vtype') qf.qf_form in
-      let trs' = 
-	if Options.notriggers || qf.qf_triggers=[] then
-	  make_triggers gopt vterm' vtype' (STRS.elements trs)
-	else 
-	  let lf = filter_good_triggers (vterm',vtype') qf.qf_triggers in
-	  if lf <> [] then lf
-	  else make_triggers gopt vterm' vtype' (STRS.elements trs)
-      in
-      let trs = 
-	STRS.filter 
-	  (fun (_,bvt,_) -> Vterm.is_empty (Vterm.inter bvt vterm')) trs in
-      let r  = {qf with qf_triggers = trs' ; qf_form = f'} in
-      (match f.c with 
-	| TFforall _ -> TFforall r , trs 
-	| _ -> TFexists r , trs)
+    | TFforall qf | TFexists qf -> 
+	let vtype' = vty_form Vtype.empty qf.qf_form in
+	let vterm' = 
+	  List.fold_left 
+	    (fun b (s,_) -> Vterm.add s b) Vterm.empty qf.qf_bvars in
+	let f', trs = 
+	  make_rec pol gopt 
+	    (Vterm.union vterm vterm') (Vtype.union vtype vtype') qf.qf_form in
+	let trs' = 
+	  if Options.notriggers || qf.qf_triggers=[] then
+	    make_triggers gopt vterm' vtype' (STRS.elements trs)
+	  else 
+	    let lf = filter_good_triggers (vterm',vtype') qf.qf_triggers in
+	    if lf <> [] then lf
+	    else make_triggers gopt vterm' vtype' (STRS.elements trs)
+	in
+	let trs = 
+	  STRS.filter 
+	    (fun (_, bvt, _) -> Vterm.is_empty (Vterm.inter bvt vterm')) trs in
+	let r  = {qf with qf_triggers = trs' ; qf_form = f'} in
+	(match f.c with 
+	   | TFforall _ -> TFforall r , trs 
+	   | _ -> TFexists r , trs)
 
-  | TFlet (up, v, t, f) ->
-      let f, trs = make_rec gopt vterm vtype f in 
-      let trs = STRS.union trs (potential_triggers (vterm, vtype) [t]) in
-       (* XXX correct for terms *)
-      TFlet (up, v, t, f), trs
+    | TFlet (up, v, t, f) ->
+	let f, trs = make_rec pol gopt vterm vtype f in 
+	let trs = STRS.union trs (potential_triggers (vterm, vtype) [t]) in
+	(* XXX correct for terms *)
+	TFlet (up, v, t, f), trs
 
-  | TFnamed(lbl, f) -> 
-      let f, trs = make_rec gopt vterm vtype f in
-      TFnamed(lbl, f), trs
+    | TFnamed(lbl, f) -> 
+	let f, trs = make_rec pol gopt vterm vtype f in
+	TFnamed(lbl, f), trs
   in
   { f with c = c }, trs
 	  
 let make gopt f = match f.c with
   | TFforall _ | TFexists _ -> 
-      let f, _ = make_rec gopt Vterm.empty Vtype.empty f in
+      let f, _ = make_rec Pos gopt Vterm.empty Vtype.empty f in
       f
   | _  ->  
       let vty = vty_form Vtype.empty f in
-      let f, trs = make_rec gopt Vterm.empty vty f in
+      let f, trs = make_rec Pos gopt Vterm.empty vty f in
       if Vtype.is_empty vty then f
       else 
 	let trs = STRS.elements trs in
