@@ -1,12 +1,14 @@
 (**************************************************************************)
 (*                                                                        *)
-(*     The Alt-ergo theorem prover                                        *)
-(*     Copyright (C) 2006-2010                                            *)
+(*     The Alt-Ergo theorem prover                                        *)
+(*     Copyright (C) 2006-2011                                            *)
 (*                                                                        *)
 (*     Sylvain Conchon                                                    *)
 (*     Evelyne Contejean                                                  *)
-(*     Stephane Lescuyer                                                  *)
+(*                                                                        *)
+(*     Francois Bobot                                                     *)
 (*     Mohamed Iguernelala                                                *)
+(*     Stephane Lescuyer                                                  *)
 (*     Alain Mebsout                                                      *)
 (*                                                                        *)
 (*     CNRS - INRIA - Universite Paris Sud                                *)
@@ -41,7 +43,7 @@ module type S = sig
 
   val are_equal : t -> Term.t -> Term.t -> Sig.answer
   val are_distinct : t -> Term.t -> Term.t -> Sig.answer
-
+  val already_distinct : t -> R.r list -> bool
   val class_of : t -> Term.t -> Term.t list
 
   val print : Format.formatter -> t -> unit
@@ -174,15 +176,17 @@ module Make ( R : Sig.X ) = struct
       MapR.iter (fun k s -> fprintf fmt "%a -> %a\n" R.print k lm_print s) m
 
     let all fmt env = 
-      fprintf fmt "------------------------------------------------------@.";
-      fprintf fmt "%a %a %a %a %a" 
-        pmake env.make 
-        prepr env.repr 
-        prules env.ac_rs 
-        pclasses env.classes
-        pneqs env.neqs;
-      fprintf fmt "------------------------------------------------------@."
-
+      if debug_uf then
+	begin
+	  fprintf fmt "-------------------------------------------------@.";
+	  fprintf fmt "%a %a %a %a %a" 
+            pmake env.make 
+            prepr env.repr 
+            prules env.ac_rs 
+            pclasses env.classes
+            pneqs env.neqs;
+	  fprintf fmt "-------------------------------------------------@."
+	end
   end
 
   
@@ -218,6 +222,7 @@ module Make ( R : Sig.X ) = struct
 	  let s = try MapR.find x gamma with Not_found -> SetR.empty in
 	  MapR.add x (SetR.add r s) gamma) gamma (R.leaves c)
 	
+    (* r1 = r2 => neqs(r1) \uplus neqs(r2) *)
     let update_neqs r1 r2 dep env = 
       let nq_r1 = lookup_for_neqs env r1 in
       let nq_r2 = lookup_for_neqs env r2 in
@@ -227,6 +232,8 @@ module Make ( R : Sig.X ) = struct
 	     try 
 	       let ex2 = MapL.find l1 mapl in
 	       let ex = Ex.union (Ex.union ex1 ex2) dep in (* VERIF *)
+	       if rules = 3 then
+		 fprintf fmt "[rule] TR-CCX-Congruence-Conflict@.";
 	       raise (Inconsistent ex)
 	     with Not_found -> 
 	       MapL.add l1 (Ex.union ex1 dep) mapl) 
@@ -250,6 +257,8 @@ module Make ( R : Sig.X ) = struct
 	      else di_un (l1,c,(b,n)::l2) ((a,m)::r,s)
       in di_un ([],[],[]) (l_1,l_2)
 
+
+    (* Debut : Code pour la mise en forme normale modulo env *)
     exception List_minus_exn
     let list_minus l_1 l_2 = 
       let rec di_un l1 l_1 l_2 = 
@@ -313,7 +322,7 @@ module Make ( R : Sig.X ) = struct
 	  else (R.color ac, R.color rac) :: z, Ex.union ex ex_ac)
         st ([], Ex.empty)
 	
-    let canon_aux rx = L.fold_left (fun r (p,v) -> R.subst p v r) rx
+    let canon_aux rx = List.fold_left (fun r (p,v) -> R.subst p v r) rx
       
     let rec canon env r ex_r = 
       let se, sac = filter_leaves r in
@@ -323,41 +332,51 @@ module Make ( R : Sig.X ) = struct
       let ex_r2 = Ex.union (Ex.union ex_r ex_subst) ex_ac in
       if R.equal r r2 then r2, ex_r2 else canon env r2 ex_r2
 
-    let canon env r =
+    let normal_form env r =
       let rr, ex = canon env r Ex.empty in
       if rewriting && verbose then 
         fprintf fmt "canon %a = %a@." R.print r R.print rr;
       rr,ex
 
-    let find_or_canon env r =
-      try MapR.find r env.repr with Not_found -> canon env r
+    (* Fin : Code pour la mise en forme normale modulo env *)
+
+
+    let find_or_normal_form env r =
+      try MapR.find r env.repr with Not_found -> normal_form env r
 
     let init_leaf env p = 
-      if MapR.mem p env.repr then env 
-      else begin
-        if debug_uf then fprintf fmt "init_leaf: %a@." R.print p;
-	{ env with
-	    repr    = MapR.add p (p, Ex.empty) env.repr;
-	    classes = update_classes p p env.classes;
-	    gamma   = add_to_gamma p p env.gamma ;
-	    neqs    = update_neqs p p Ex.empty env } 
-      end
+      if debug_uf then fprintf fmt "init_leaf: %a@." R.print p;
+      let in_repr = MapR.mem p env.repr in
+      let in_neqs = MapR.mem p env.neqs in
+      { env with
+	repr    = 
+	  if in_repr then env.repr 
+	  else MapR.add p (p, Ex.empty) env.repr;
+	classes = 
+	  if in_repr then env.classes
+	  else update_classes p p env.classes;
+	gamma   = 
+	  if in_repr then env.gamma
+	  else add_to_gamma p p env.gamma ;
+	neqs    =  
+	  if in_neqs then env.neqs 
+	  else update_neqs p p Ex.empty env } 
           
     let init_term env t = 
       let mkr, ctx = R.make t in
-      let rp, ex = canon env mkr in
+      let rp, ex = normal_form env mkr in
       {env with
 	make    = MapT.add t mkr env.make; 
 	repr    = MapR.add mkr (rp,ex) env.repr;
 	classes = add_to_classes t rp env.classes;
 	gamma   = add_to_gamma mkr rp env.gamma;
 	neqs    = 
-	  if MapR.mem rp env.neqs then env.neqs 
+	  if MapR.mem rp env.neqs then env.neqs (* pourquoi ce test *)
 	  else MapR.add rp MapL.empty env.neqs}, ctx
 
 
     let head_cp eqs env (({h=h} as ac), v, dep) = 
-      if RS.mem h env.ac_rs then
+      try (*if RS.mem h env.ac_rs then*)
         SetRL.iter
 	  (fun (g, d, dep_rl) ->
 	     match disjoint_union ac.l g.l with
@@ -369,9 +388,9 @@ module Make ( R : Sig.X ) = struct
                      fprintf fmt "[uf] critical pair: %a = %a@." 
                        R.print rx R.print ry;
                    if not (R.equal rx ry) then 
-                     Queue.push (rx, ry, Ex.union dep dep_rl) eqs
-	  )(RS.find h env.ac_rs)
-
+                     Queue.push (rx, ry, Ex.union dep dep_rl) eqs)
+	  (RS.find h env.ac_rs)
+      with Not_found -> assert false
 	
     let comp_collapse eqs env (p, v, dep) = 
       RS.fold
@@ -380,11 +399,17 @@ module Make ( R : Sig.X ) = struct
 	    (fun ((g, d, dep_rl) as rul) env ->
 	      let env = {env with ac_rs = RS.remove_rule rul env.ac_rs} in
 	      let gx = R.color g in
-	      let g2, ex_g2 = canon env (Ac.subst p v g) in
-	      let d2, ex_d2 = canon env (R.subst p v d) in
+	      let g2, ex_g2 = normal_form env (Ac.subst p v g) in
+	      let d2, ex_d2 = normal_form env (R.subst p v d) in
 	      if R.equal g2 gx then (* compose *)
-                let ex = Ex.union ex_d2 (Ex.union dep_rl dep) in
-	        {env with ac_rs = RS.add_rule (g,d2, ex) env.ac_rs}
+		begin
+		  if R.equal g2 d2 then
+		    Format.eprintf "Compose : %a -> %a sur %a et %a@." 
+		      R.print p R.print v
+		      Ac.print g R.print d;
+                  let ex = Ex.union ex_d2 (Ex.union dep_rl dep) in
+	          {env with ac_rs = RS.add_rule (g,d2, ex) env.ac_rs}
+		  end
 	      else (* collapse *)
                 begin
                   if debug_ac then
@@ -445,7 +470,7 @@ module Make ( R : Sig.X ) = struct
       else
 	let env, tch, neqs_to_up = MapR.fold
 	  (fun r (rr,ex) (env,tch,neqs_to_up) ->
-	     let nrr, ex_nrr = canon env rr in
+	     let nrr, ex_nrr = normal_form env rr in
 	     if R.equal nrr rr then env, tch, neqs_to_up
 	     else 
 	       let ex = Ex.union ex ex_nrr in
@@ -459,8 +484,6 @@ module Make ( R : Sig.X ) = struct
         in 
         (* Correction : Do not update neqs twice for the same r *)
 	update_aux dep neqs_to_up env, tch 
-	
-
 	  
     let apply_sigma eqs env tch ((p, v, dep) as sigma) = 
       let env = init_leaf env p in
@@ -471,29 +494,37 @@ module Make ( R : Sig.X ) = struct
   end
     
   let add env t = 
+    if rules = 3 then fprintf fmt "[rule] TR-UFX-Add@.";
     if MapT.mem t env.make then env, [] else Env.init_term env t
 
   let ac_solve eqs dep (env, tch) (p, v) = 
+    (* pourquoi recuperer le representant de rv? r = rv d'apres testopt *)
     if debug_uf then 
       printf "[uf] ac-solve: %a |-> %a %a@." R.print p R.print v Ex.print dep;
-    assert ( let rp, _ = Env.find_or_canon env p in R.equal p rp);
-    let rv, ex_rv = Env.find_or_canon env v in
+    assert ( let rp, _ = Env.find_or_normal_form env p in R.equal p rp);
+    let rv, ex_rv = Env.find_or_normal_form env v in
+    assert ( let rv, _ = Env.find_or_normal_form env v in R.equal v rv);
     let dep = Ex.union ex_rv dep in
     Env.apply_sigma eqs env tch (p, rv, dep)
 
   let x_solve env r1 r2 dep = 
-    let rr1, ex_r1 = Env.find_or_canon env r1 in
-    let rr2, ex_r2 = Env.find_or_canon env r2 in
+    let rr1, ex_r1 = Env.find_or_normal_form env r1 in
+    let rr2, ex_r2 = Env.find_or_normal_form env r2 in
     let dep = Ex.union dep (Ex.union ex_r1 ex_r2) in
     if debug_uf then 
       printf "[uf] x-solve: %a = %a %a@."
 	R.print rr1 R.print rr2 Ex.print dep;
-    if R.equal rr1 rr2 then [] (* Remove rule *)
+    if R.equal rr1 rr2 then begin
+      if rules = 3 then fprintf fmt "[rule] TR-CCX-Remove@.";
+      [] (* Remove rule *)
+    end
     else 
       begin
 	ignore (Env.update_neqs rr1 rr2 dep env);
         try R.solve rr1 rr2 
-	with Unsolvable -> raise (Inconsistent dep)
+	with Unsolvable ->
+	  if rules = 3 then fprintf fmt "[rule] TR-CCX-Congruence-Conflict@.";
+	  raise (Inconsistent dep)
       end
         
   let rec ac_x eqs env tch = 
@@ -509,35 +540,40 @@ module Make ( R : Sig.X ) = struct
       ac_x eqs env tch
       
   let union env r1 r2 dep =
+    if rules = 3 then fprintf fmt "[rule] TR-UFX-Union@.";
     let equations = Queue.create () in 
     Queue.push (r1,r2, dep) equations;
     ac_x equations env []
 
   let rec distinct env rl dep =
+    Print.all fmt env;
     let d = Lit.make (Literal.Distinct (false,rl)) in
     if debug_uf then fprintf fmt "[uf] distinct %a@." Lit.print d;
-    let neqs, _, newds = 
+    let env, _, newds = 
       List.fold_left
-	(fun (neqs, mapr, newds) r -> 
-	   let rr, ex = Env.find_or_canon env r in 
-	   try 
-	     raise (Inconsistent (Ex.union ex (MapR.find rr mapr)))
+	(fun (env, mapr, newds) r -> 
+	   let rr, ex = Env.find_or_normal_form env r in 
+	   try
+	     let exr = MapR.find rr mapr in
+	     if rules = 3 then fprintf fmt "[rule] TR-CCX-Distinct-Conflict@.";
+	     raise (Inconsistent (Ex.union ex exr))
 	   with Not_found ->
 	     let uex = Ex.union ex dep in
 	     let mdis = 
-	       try MapR.find rr neqs with Not_found -> MapL.empty in
+	       try MapR.find rr env.neqs with Not_found -> MapL.empty in
 	     let mdis = 
 	       try 
 		 MapL.add d (Ex.merge uex (MapL.find d mdis)) mdis
 	       with Not_found -> 
 		 MapL.add d uex mdis
 	     in
-	     MapR.add rr mdis neqs, MapR.add rr uex mapr, (rr, ex, mapr)::newds
+	     let env = Env.init_leaf env rr in
+             let env = {env with neqs = MapR.add rr mdis env.neqs} in
+             env, MapR.add rr uex mapr, (rr, ex, mapr)::newds
 	)
-	(env.neqs, MapR.empty, [])
+	(env, MapR.empty, [])
 	rl
     in
-    let env = { env with neqs = neqs } in
     List.fold_left 
       (fun env (r1, ex1, mapr) -> 
 	 MapR.fold (fun r2 ex2 env -> 
@@ -548,10 +584,14 @@ module Make ( R : Sig.X ) = struct
 			      (R.equal a r2 && R.equal b r1) then env
 			    else
 			      distinct env [a; b] ex
-			| []  -> raise (Inconsistent ex) 
+			| []  -> 
+			  if rules = 3 then
+			    fprintf fmt "[rule] TR-CCX-Distinct-Conflict@.";
+			  raise (Inconsistent ex) 
 			| _   -> env
 		      with Unsolvable -> env) mapr env)
       env newds
+
 			  
   let are_equal env t1 t2 = 
     let r1, ex_r1 = Env.lookup_by_t t1 env in
@@ -568,15 +608,29 @@ module Make ( R : Sig.X ) = struct
       No
     with Inconsistent ex -> Yes(ex)
 
+  let already_distinct env lr = 
+    let d = Lit.make (Literal.Distinct (false,lr)) in
+    try 
+      List.iter (fun r -> 
+	let mdis = MapR.find r env.neqs in
+	ignore (MapL.find d mdis)
+      ) lr;
+      true
+    with Not_found -> false
+
   let class_of env t = 
     try 
       let rt, _ = MapR.find (MapT.find t env.make) env.repr in
       SetT.elements (MapR.find rt env.classes)
     with Not_found -> [t]
       
-  let find env t = Env.lookup_by_t t env
+  let find env t = 
+    if rules = 3 then fprintf fmt "[rule] TR-UFX-Find@.";
+    Env.lookup_by_t t env
 
-  let find_r = Env.find_or_canon
+  let find_r = 
+    if rules = 3 then fprintf fmt "[rule] TR-UFX-Find@.";
+    Env.find_or_normal_form
 
   let print = Print.all 
 
