@@ -1,3 +1,22 @@
+(**************************************************************************)
+(*                                                                        *)
+(*     The Alt-Ergo theorem prover                                        *)
+(*     Copyright (C) 2006-2011                                            *)
+(*                                                                        *)
+(*     Sylvain Conchon                                                    *)
+(*     Evelyne Contejean                                                  *)
+(*                                                                        *)
+(*     Francois Bobot                                                     *)
+(*     Mohamed Iguernelala                                                *)
+(*     Stephane Lescuyer                                                  *)
+(*     Alain Mebsout                                                      *)
+(*                                                                        *)
+(*     CNRS - INRIA - Universite Paris Sud                                *)
+(*                                                                        *)
+(*   This file is distributed under the terms of the CeCILL-C licence     *)
+(*                                                                        *)
+(**************************************************************************)
+
 open Why_annoted
 
 open Lexing
@@ -25,10 +44,18 @@ let prune_nodep r t =
   r.pruned <- true;
   t#set_property (`FOREGROUND "light gray")
 
+let incorrect_prune_nodep r t =
+  r.pruned <- true;
+  t#set_property (`FOREGROUND "tomato")
+
 let unprune_nodep r t =
   r.pruned <- false;
   t#set_property (`FOREGROUND_SET false)
 
+let toggle_incorrect_prune r t =
+  if r.pruned then unprune_nodep r t
+  else incorrect_prune_nodep r t
+
 let toggle_prune_nodep r t =
   if r.pruned then unprune_nodep r t
   else prune_nodep r t
@@ -42,7 +69,7 @@ let search_using t sbuf env =
 	  let tags = findtags_using r.c env.ast in
 	  env.search_tags <- tags;
 	  List.iter (fun t -> t#set_property (`BACKGROUND "gold")) tags
-	| AT {c = at} | AF {c = AFatom (AApred at)} ->
+	| AT {c = at} | AF ({c = AFatom (AApred at)}, _) ->
 	  let tags = findtags_dep at env.ast in
 	  env.search_tags <- tags;
 	  List.iter (fun t -> t#set_property (`BACKGROUND "orange")) tags
@@ -89,7 +116,14 @@ let tag_callback t env sbuf ~origin:y z i =
 		    if r.pruned then unprune r t env.dep 
 		    else prune r t env.dep
 		  else toggle_prune_nodep r t
-		| AF r -> toggle_prune_nodep r t
+		| AF (r, Some parent) ->
+		  begin match parent.c, parent.polarity with
+		    | AFop (AOPand, _), false | AFop (AOPor, _), true
+		    | AFop (AOPimp, _), true | AFop (AOPiff, _), _ ->
+		        toggle_incorrect_prune r t
+		    | _ -> toggle_prune_nodep r t
+		  end
+		| AF (r, None) -> toggle_prune_nodep r t
 		| AT r -> toggle_prune_nodep r t
 		| QF _ -> ()
 	end;
@@ -144,6 +178,25 @@ let term_callback t env sbuf ~origin:y z i =
       | _ -> false
 	  
 
+let rec list_uquant_vars_in_form = function
+  | AFatom _ -> []
+  | AFop (op, aafl) ->
+      List.fold_left (fun l aaf -> l@(list_uquant_vars_in_form aaf.c)) [] aafl
+  | AFforall aqf ->
+      let l = list_uquant_vars_in_form aqf.c.aqf_form.c in
+      if aqf.polarity then
+	aqf.c.aqf_bvars@l
+      else l
+  | AFexists aqf ->
+      let l = list_uquant_vars_in_form aqf.c.aqf_form.c in
+      if not aqf.polarity then
+	aqf.c.aqf_bvars@l
+      else l
+  | AFlet (upvars, s, at, aaf) ->
+      list_uquant_vars_in_form aaf.c
+  | AFnamed (_, aaf) ->
+      list_uquant_vars_in_form aaf.c
+
 let rec list_vars_in_form = function
   | AFatom _ -> []
   | AFop (op, aafl) ->
@@ -160,7 +213,7 @@ let rec is_quantified_term vars at =
   | ATconst _ -> false
   | ATvar s ->
       List.fold_left 
-	(fun b (s',_) -> b && (not (Symbols.equal s s'))) true vars
+	(fun b (s',_) -> b || (Symbols.equal s s')) false vars
   | ATapp (_, atl) ->
       List.fold_left
 	(fun b at -> b || is_quantified_term vars at) false atl
@@ -169,7 +222,7 @@ let rec is_quantified_term vars at =
   | ATinfix (at1, _, at2) ->
       is_quantified_term vars at1
       || is_quantified_term vars at2
-  | ATprefix (_, at) ->
+  | ATdot (at, _) | ATprefix (_, at) ->
       is_quantified_term vars at
   | ATextract (at1, at2, at3)
   | ATset (at1, at2, at3) ->
@@ -181,13 +234,9 @@ let rec is_quantified_term vars at =
 	List.filter (fun (s'',_) -> not (Symbols.equal s' s'')) vars in
       is_quantified_term vars at1
       || is_quantified_term nvars at2
-
-let rec remove_doublons = function
-  | [] -> []
-  | a::r ->
-      if List.mem a r then remove_doublons r
-      else a::(remove_doublons r)
-
+  | ATrecord r ->
+      List.fold_left
+	(fun b (_, at) -> b || is_quantified_term vars at) false r
 
 let unquantify_aaterm (buffer:sbuffer) at =
   new_annot buffer at.c (Why_typing.new_id ()) (tag buffer)
@@ -202,48 +251,7 @@ let unquantify_aatom (buffer:sbuffer) = function
   | AAlt aatl -> AAlt (List.map (unquantify_aaterm buffer) aatl)
   | AApred a -> AApred a
   | AAbuilt (h,aatl) -> AAbuilt (h, (List.map (unquantify_aaterm buffer) aatl))
-
-let rec unquantify_aform (buffer:sbuffer) vars f =
-  let ptag = (tag buffer) in
-  let c = match f with
-    | AFatom aa -> AFatom (unquantify_aatom buffer aa)
-    | AFop (op, afl) ->
-      AFop (op, List.map
-	(fun af -> unquantify_aform buffer vars af.c) afl)
-    | AFforall aaqf | AFexists aaqf ->
-      let {aqf_bvars = bv; aqf_upvars = uv; aqf_triggers = atll; aqf_form = af}=
-	aaqf.c in
-      let aqf_bvars = List.filter (fun v -> not (List.mem v vars)) bv in
-      let aform = unquantify_aform buffer vars af.c in
-      if aqf_bvars = [] then aform.c
-      else 
-	let aqf_triggers = 
-	  List.map (List.map (unquantify_aaterm buffer)) atll in
-	let aqf_triggers = List.filter
-	  (fun aatl ->	   
-	    List.filter (fun aat -> is_quantified_term vars aat.c) aatl <> []
-	  ) aqf_triggers in
-	if aqf_triggers = [] then aform.c
-	else let c =
-	       { aqf_bvars = List.filter (fun v -> not (List.mem v vars)) bv;
-		 aqf_upvars = List.filter (fun v -> not (List.mem v vars)) uv;
-		 aqf_triggers =  aqf_triggers;
-		 aqf_form = aform} in
-	     (match f with
-	       | AFforall _ -> 
-		 AFforall (new_annot buffer c (Why_typing.new_id ()) (tag buffer))
-	       | AFexists _ -> 
-		 AFexists (new_annot buffer c (Why_typing.new_id ()) (tag buffer))
-	       | _ -> assert false)
-    | AFlet (uv, s, at, aaf) ->
-      AFlet (List.filter (fun v -> not (List.mem v vars)) uv, s, at,
-	     unquantify_aform buffer vars aaf.c)
-    | AFnamed (n, aaf) ->
-      (unquantify_aform buffer vars aaf.c).c
-  in
-  new_annot buffer c (Why_typing.new_id ()) ptag
-      
-  
+         
 
 let rec aterm_used_vars goal_vars at =
   match at.at_desc with
@@ -253,43 +261,154 @@ let rec aterm_used_vars goal_vars at =
 	 with Not_found ->  [])
     | ATapp (_, atl) ->
 	List.fold_left (fun l at -> aterm_used_vars goal_vars at @ l) [] atl
-    | ATprefix (_, at) | ATlet (_, _, at) -> aterm_used_vars goal_vars at
+    | ATdot (at, _) | ATprefix (_, at) | ATlet (_, _, at) ->
+	aterm_used_vars goal_vars at
     | ATinfix (at1, _, at2) | ATget (at1, at2) | ATconcat (at1, at2) ->
 	(aterm_used_vars goal_vars at1)@(aterm_used_vars goal_vars at2)
     | ATset (at1, at2, at3) | ATextract (at1, at2, at3) ->
 	(aterm_used_vars goal_vars at1)@
 	  (aterm_used_vars goal_vars at2)@
 	  (aterm_used_vars goal_vars at3)
+    | ATrecord r ->
+	List.fold_left (fun l (_, at) -> aterm_used_vars goal_vars at @ l) [] r
+
+
+let rec unquantify_aform (buffer:sbuffer) tyenv vars_entries 
+    used_vars goal_vars f pol =
+  let ptag = (tag buffer) in
+  let c, ve, goal_used = match f, pol with
+
+    | AFatom aa, _ -> 
+      AFatom (unquantify_aatom buffer aa), vars_entries, []
+
+    | AFop (op, afl), _ ->
+      let nafl, ve, goal_used = 
+	List.fold_left (fun (nafl, ve, gu) af ->
+	  let res, ve, gu' = unquantify_aform buffer tyenv ve used_vars 
+	    goal_vars af.c af.polarity 
+	in
+	(res::nafl, ve, gu'@gu))
+	([], vars_entries, []) afl in
+      AFop (op, List.rev nafl), ve, goal_used
+
+    | AFforall aaqf, true | AFexists aaqf, false ->
+      let {aqf_bvars = bv; aqf_upvars = uv; aqf_triggers = atll; aqf_form = af}=
+	aaqf.c in
+      let nbv, used, goal_used, ve, _, lets =
+	List.fold_left (fun (nbv, used, goal_used, ve, uplet, lets) v ->
+	  let ((s, _) as v'), e = List.hd ve in
+	  let cdr_ve = List.tl ve in
+	  assert (v = v');
+	  if e#text = "" then 
+	    (v'::nbv, used, goal_used, cdr_ve, v'::uplet, lets)
+	  else
+	    let lb = Lexing.from_string e#text in
+	    let lexpr = Why_parser.lexpr Why_lexer.token lb in
+	    let at, gu =
+	      try 
+		let tt = Why_typing.term tyenv uplet lexpr in
+		annot_of_tterm buffer tt, []
+	      with Common.Error _ ->
+		let gv = List.fold_left (fun acc v ->
+		  if List.mem v uplet then acc
+		  else v::acc) [] goal_vars
+		in
+		let tt = Why_typing.term tyenv (uplet@gv) lexpr in
+		let at = annot_of_tterm buffer tt in
+		at, aterm_used_vars gv at.c
+	    in
+	    (nbv, v'::used, gu@goal_used, cdr_ve, 
+	     v'::uplet, (uplet, s, at)::lets))
+	  ([], [], [], vars_entries, uv, []) bv in
+      let aform, ve, gu =
+	unquantify_aform buffer tyenv ve used goal_vars af.c af.polarity
+      in
+      let goal_used = gu@goal_used in
+      let add_lets afc lets =
+	List.fold_left
+	  (fun af (u, s, at) -> 
+	    new_annot buffer (AFlet (u, s, at.c, af))
+	      (Why_typing.new_id ()) (tag buffer))
+	  afc lets in
+      if nbv = [] then (add_lets aform lets).c, ve, goal_used
+      else 
+	let aqf_triggers = 
+	  List.map (List.map (unquantify_aaterm buffer)) atll in
+	let aqf_triggers = List.filter
+	  (fun aatl ->
+	    (* TODO : change nbv with something else *)
+	    List.filter (fun aat -> is_quantified_term nbv aat.c) aatl <> []
+	  ) aqf_triggers in
+	if aqf_triggers = [] then (add_lets aform lets).c, ve, goal_used
+	else 
+	  let c =
+	    { aqf_bvars = nbv;
+	      aqf_upvars = List.filter (fun v -> not (List.mem v used_vars)) uv;
+	      aqf_triggers =  aqf_triggers;
+	      aqf_form =  add_lets aform lets} in
+	  (match f with
+	    | AFforall _ -> 
+	      AFforall (new_annot buffer c (Why_typing.new_id ()) (tag buffer)),
+	      ve, goal_used
+	    | AFexists _ -> 
+	      AFexists (new_annot buffer c (Why_typing.new_id ()) (tag buffer)),
+	      ve, goal_used
+	    | _ -> assert false)
+
+    | AFforall aaqf, false | AFexists aaqf, true ->
+      let naqf_form, ve, goal_used =
+	unquantify_aform buffer tyenv vars_entries used_vars goal_vars
+	aaqf.c.aqf_form.c aaqf.c.aqf_form.polarity in
+      let c = { aaqf.c with aqf_form = naqf_form } in
+      (match f with
+	| AFforall _ -> 
+	  AFforall (new_annot buffer c (Why_typing.new_id ()) (tag buffer)),
+	  ve, goal_used
+	| AFexists _ -> 
+	  AFexists (new_annot buffer c (Why_typing.new_id ()) (tag buffer)),
+	  ve, goal_used
+	| _ -> assert false)
+
+    | AFlet (uv, s, at, aaf), _ ->
+      let naaf, ve, goal_used = 
+	unquantify_aform buffer tyenv vars_entries used_vars goal_vars
+	  aaf.c aaf.polarity
+      in
+      AFlet (List.filter (fun v -> not (List.mem v used_vars)) uv, s, at, naaf),
+      ve, goal_used
+
+    | AFnamed (n, aaf), _ ->
+      let naaf, ve, goal_used =
+	unquantify_aform buffer tyenv vars_entries used_vars goal_vars
+	  aaf.c aaf.polarity 
+      in
+      AFnamed (n, naaf), ve, goal_used
+  in
+  new_annot buffer c (Why_typing.new_id ()) ptag, ve, goal_used
 
 
 let make_instance (buffer:sbuffer) vars (entries:GEdit.entry list)
     afc goal_form tyenv =
   let goal_vars = list_vars_in_form goal_form.c in
-  let terms, used_vars, vars = List.fold_left2
-    (fun (l,u,vl) e v ->
-       if e#text <> "" then
-	 let lb = Lexing.from_string e#text in
-	 let lexpr = Why_parser.lexpr Why_lexer.token lb in
-	 let tt = Why_typing.term tyenv goal_vars lexpr in
-	 let at = annot_of_tterm buffer tt in
-	 let used_vars = aterm_used_vars goal_vars at.c in
-	 at::l, (remove_doublons used_vars)::u, v::vl
-       else l, u, vl
-    ) ([],[],[]) entries (List.rev vars) in
-  let aform = List.fold_left2
-    (fun af (s, ty) (at, u) -> 
-      new_annot buffer (AFlet (u, s, at.c, af)) (Why_typing.new_id ()) (tag buffer))
-    (unquantify_aform buffer vars afc) vars (List.combine terms used_vars)
+  if debug then List.iter (fun (v,e) ->
+    eprintf "%a -> %s@." Symbols.print (fst v) e#text)
+    (List.combine vars (List.rev entries));
+  let aform, _, goal_used = 
+    unquantify_aform buffer tyenv (List.combine vars (List.rev entries)) [] 
+      goal_vars afc true
   in
-  let all_used_vars = remove_doublons (List.flatten used_vars) in
-  (* new_annot buffer *) aform, all_used_vars
+  aform, goal_used
   
 
+
+
 exception UncoveredVar of (Symbols.t * Ty.t)
 
+type nestedq = Forall of aform annoted | Exists of aform annoted
+
 let rec least_nested_form used_vars af =
   match used_vars, af.c with
-    | [], _ -> af
+    | [], _ -> Exists af
     | v::r, AFatom _ -> raise(UncoveredVar v)
     | v::r, AFop (op, aafl) ->
 	let rec least_list = function
@@ -298,12 +417,19 @@ let rec least_nested_form used_vars af =
 	       try least_nested_form used_vars af
 	       with UncoveredVar _ -> least_list l
 	in least_list aafl
-    | _, AFforall aqf | _, AFexists aqf ->
+    | _, AFforall aqf ->
 	let not_covered = List.fold_left
 	  (fun l v ->
 	     if List.mem v aqf.c.aqf_bvars then l else v::l (*XXX*)
 	  ) [] used_vars in
-	if not_covered = [] then af
+	if not_covered = [] then Forall aqf.c.aqf_form
+	else least_nested_form not_covered aqf.c.aqf_form
+    | _, AFexists aqf ->
+	let not_covered = List.fold_left
+	  (fun l v ->
+	     if List.mem v aqf.c.aqf_bvars then l else v::l (*XXX*)
+	  ) [] used_vars in
+	if not_covered = [] then Exists aqf.c.aqf_form
 	else least_nested_form not_covered aqf.c.aqf_form
     | _, AFlet (upvars, s, at, af) ->
 	least_nested_form used_vars af
@@ -326,7 +452,7 @@ let rec add_instance env vars entries af aname =
     make_instance env.inst_buffer vars entries af goal_form tyenv in
   let ln_form = least_nested_form used_vars goal_form in
   env.inst_buffer#place_cursor  ~where:env.inst_buffer#end_iter;
-  if ln_form = goal_form then begin
+  if ln_form = Exists goal_form then begin
     let hy = AAxiom (loc, (sprintf "%s%s" "_instance_" aname), instance.c) in
     let ahy = new_annot env.inst_buffer hy instance.id ptag in
     let rev_ast = List.rev env.ast in
@@ -341,10 +467,18 @@ let rec add_instance env vars entries af aname =
   end
   else begin
     let instance = new_annot env.inst_buffer instance.c instance.id ptag in
-    ln_form.c <- 
-      AFop 
-      (AOPimp, 
-       [instance; {ln_form with c = ln_form.c}]);
+    begin match ln_form with 
+      | Exists lnf ->
+	lnf.c <- 
+	  AFop 
+	  (AOPand, 
+	   [instance; {lnf with c = lnf.c}])
+      | Forall lnf ->
+	lnf.c <- 
+	  AFop 
+	  (AOPimp, 
+	   [instance; {lnf with c = lnf.c}])
+    end;
     env.inst_buffer#insert ~tags:[instance.tag] ("instance "^aname^": \n");
     connect_aaform env env.inst_buffer instance;
     env.inst_buffer#insert (String.make indent_size ' ');
@@ -356,7 +490,7 @@ let rec add_instance env vars entries af aname =
 
 and popup_axiom t env offset () =
     let pop_w = GWindow.dialog
-    ~title:"Instanciate axiom"
+    ~title:"Instantiate axiom"
     ~allow_grow:true
     ~width:400 ()
     (* ~icon:(GdkPixbuf.from_xpm_data Logo.xpm_logo) ()  *)
@@ -380,16 +514,16 @@ and popup_axiom t env offset () =
       begin
 	  match atd.c with
 	    | AAxiom (_, aname, af) ->
-		pop_w#set_title ("Instanciate axiom "^aname)
+		pop_w#set_title ("Instantiate axiom "^aname)
 	    | APredicate_def (_, aname,_ , af) ->
-		pop_w#set_title ("Instanciate predicate "^aname)
+		pop_w#set_title ("Instantiate predicate "^aname)
 	    | _ -> assert false
       end;
       begin
 	  match atd.c with
 	    | AAxiom (_, aname, af)
 	    | APredicate_def (_, aname,_ , af) ->
-		let vars = remove_doublons (list_vars_in_form af) in
+		let vars = list_uquant_vars_in_form af in
 		let rows = List.length vars in
 		let table = GPack.table ~rows ~columns:2 ~homogeneous:false
 		  ~border_width:5 ~packing:pop_w#vbox#add () in
@@ -628,11 +762,14 @@ and connect_at_desc env sbuf = function
     | ATconcat (t1, t2) | ATlet (_, t1, t2) ->
 	connect_aterm env sbuf t1;
 	connect_aterm env sbuf t2
-    | ATprefix (_, t) -> connect_aterm env sbuf t
+    | ATdot (t, _) | ATprefix (_, t) -> connect_aterm env sbuf t
     | ATset (t1,t2,t3) | ATextract (t1,t2,t3) ->
 	connect_aterm env sbuf t1;
 	connect_aterm env sbuf t2;
 	connect_aterm env sbuf t3
+    | ATrecord r -> 
+	let atl = List.map snd r in
+	connect_aterm_list env sbuf atl
 	
 and connect_aatom env sbuf aa =
   match aa with
@@ -713,6 +850,8 @@ let show_used_lemmas env expl =
   env.proof_tags <- ftags;
   env.proof_toptags <- atags
   
+
+(* More efficient but invariant broken when using user instanciated axioms
 let prune_unused env expl =
   let ids = match Explanation.ids_of expl with
     | None -> []
@@ -738,3 +877,15 @@ let prune_unused env expl =
       	  end
   in
   aux false env.ast ids
+*)
+
+let prune_unused env =
+  let prune_top d = match d.c with
+    | ATypeDecl _ | AGoal _ | ALogic _ -> ()
+    | _ -> prune_nodep d d.tag
+  in
+  List.iter (fun (d, _) -> 
+    if not (List.mem d.ptag env.proof_toptags) 
+      && not (List.mem d.ptag env.proof_tags) 
+    then prune_top d
+  ) env.ast
